package com.sun.java.util.jar.pack;

import java.io.FilterOutputStream;
import java.io.FilterInputStream;
import java.io.EOFException;
import java.io.ByteArrayInputStream;
import java.lang.reflect.Array;
import java.io.PrintStream;
import java.io.InputStream;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.ByteArrayOutputStream;
import java.util.Arrays;
import java.util.Iterator;
import java.io.OutputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.ArrayList;
import java.io.IOException;
import java.util.LinkedList;
import java.io.File;
import java.util.List;
import java.util.Map;

abstract class BandStructure
{
    static final int MAX_EFFORT = 9;
    static final int MIN_EFFORT = 1;
    static final int DEFAULT_EFFORT = 5;
    PropMap p200;
    int verbose;
    int effort;
    boolean optDumpBands;
    boolean optDebugBands;
    boolean optVaryCodings;
    boolean optBigStrings;
    private Package.Version highestClassVersion;
    private final boolean isReader;
    static final Coding BYTE1;
    static final Coding CHAR3;
    static final Coding BCI5;
    static final Coding BRANCH5;
    static final Coding UNSIGNED5;
    static final Coding UDELTA5;
    static final Coding SIGNED5;
    static final Coding DELTA5;
    static final Coding MDELTA5;
    private static final Coding[] basicCodings;
    private static final Map<Coding, Integer> basicCodingIndexes;
    protected byte[] bandHeaderBytes;
    protected int bandHeaderBytePos;
    protected int bandHeaderBytePos0;
    static final int SHORT_BAND_HEURISTIC = 100;
    public static final int NO_PHASE = 0;
    public static final int COLLECT_PHASE = 1;
    public static final int FROZEN_PHASE = 3;
    public static final int WRITE_PHASE = 5;
    public static final int EXPECT_PHASE = 2;
    public static final int READ_PHASE = 4;
    public static final int DISBURSE_PHASE = 6;
    public static final int DONE_PHASE = 8;
    private final List<BandStructure.CPRefBand> allKQBands;
    private List<Object[]> needPredefIndex;
    private CodingChooser codingChooser;
    static final byte[] defaultMetaCoding;
    static final byte[] noMetaCoding;
    BandStructure.ByteCounter outputCounter;
    protected int archiveOptions;
    protected long archiveSize0;
    protected long archiveSize1;
    protected int archiveNextCount;
    static final int AH_LENGTH_0 = 3;
    static final int AH_LENGTH_MIN = 15;
    static final int AH_LENGTH_S = 2;
    static final int AH_ARCHIVE_SIZE_HI = 0;
    static final int AH_ARCHIVE_SIZE_LO = 1;
    static final int AH_FILE_HEADER_LEN = 5;
    static final int AH_SPECIAL_FORMAT_LEN = 2;
    static final int AH_CP_NUMBER_LEN = 4;
    static final int AH_CP_EXTRA_LEN = 4;
    static final int AB_FLAGS_HI = 0;
    static final int AB_FLAGS_LO = 1;
    static final int AB_ATTR_COUNT = 2;
    static final int AB_ATTR_INDEXES = 3;
    static final int AB_ATTR_CALLS = 4;
    private static final boolean NULL_IS_OK = true;
    BandStructure.MultiBand all_bands;
    BandStructure.ByteBand archive_magic;
    BandStructure.IntBand archive_header_0;
    BandStructure.IntBand archive_header_S;
    BandStructure.IntBand archive_header_1;
    BandStructure.ByteBand band_headers;
    BandStructure.MultiBand cp_bands;
    BandStructure.IntBand cp_Utf8_prefix;
    BandStructure.IntBand cp_Utf8_suffix;
    BandStructure.IntBand cp_Utf8_chars;
    BandStructure.IntBand cp_Utf8_big_suffix;
    BandStructure.MultiBand cp_Utf8_big_chars;
    BandStructure.IntBand cp_Int;
    BandStructure.IntBand cp_Float;
    BandStructure.IntBand cp_Long_hi;
    BandStructure.IntBand cp_Long_lo;
    BandStructure.IntBand cp_Double_hi;
    BandStructure.IntBand cp_Double_lo;
    BandStructure.CPRefBand cp_String;
    BandStructure.CPRefBand cp_Class;
    BandStructure.CPRefBand cp_Signature_form;
    BandStructure.CPRefBand cp_Signature_classes;
    BandStructure.CPRefBand cp_Descr_name;
    BandStructure.CPRefBand cp_Descr_type;
    BandStructure.CPRefBand cp_Field_class;
    BandStructure.CPRefBand cp_Field_desc;
    BandStructure.CPRefBand cp_Method_class;
    BandStructure.CPRefBand cp_Method_desc;
    BandStructure.CPRefBand cp_Imethod_class;
    BandStructure.CPRefBand cp_Imethod_desc;
    BandStructure.IntBand cp_MethodHandle_refkind;
    BandStructure.CPRefBand cp_MethodHandle_member;
    BandStructure.CPRefBand cp_MethodType;
    BandStructure.CPRefBand cp_BootstrapMethod_ref;
    BandStructure.IntBand cp_BootstrapMethod_arg_count;
    BandStructure.CPRefBand cp_BootstrapMethod_arg;
    BandStructure.CPRefBand cp_InvokeDynamic_spec;
    BandStructure.CPRefBand cp_InvokeDynamic_desc;
    BandStructure.MultiBand attr_definition_bands;
    BandStructure.ByteBand attr_definition_headers;
    BandStructure.CPRefBand attr_definition_name;
    BandStructure.CPRefBand attr_definition_layout;
    BandStructure.MultiBand ic_bands;
    BandStructure.CPRefBand ic_this_class;
    BandStructure.IntBand ic_flags;
    BandStructure.CPRefBand ic_outer_class;
    BandStructure.CPRefBand ic_name;
    BandStructure.MultiBand class_bands;
    BandStructure.CPRefBand class_this;
    BandStructure.CPRefBand class_super;
    BandStructure.IntBand class_interface_count;
    BandStructure.CPRefBand class_interface;
    BandStructure.IntBand class_field_count;
    BandStructure.IntBand class_method_count;
    BandStructure.CPRefBand field_descr;
    BandStructure.MultiBand field_attr_bands;
    BandStructure.IntBand field_flags_hi;
    BandStructure.IntBand field_flags_lo;
    BandStructure.IntBand field_attr_count;
    BandStructure.IntBand field_attr_indexes;
    BandStructure.IntBand field_attr_calls;
    BandStructure.CPRefBand field_ConstantValue_KQ;
    BandStructure.CPRefBand field_Signature_RS;
    BandStructure.MultiBand field_metadata_bands;
    BandStructure.MultiBand field_type_metadata_bands;
    BandStructure.CPRefBand method_descr;
    BandStructure.MultiBand method_attr_bands;
    BandStructure.IntBand method_flags_hi;
    BandStructure.IntBand method_flags_lo;
    BandStructure.IntBand method_attr_count;
    BandStructure.IntBand method_attr_indexes;
    BandStructure.IntBand method_attr_calls;
    BandStructure.IntBand method_Exceptions_N;
    BandStructure.CPRefBand method_Exceptions_RC;
    BandStructure.CPRefBand method_Signature_RS;
    BandStructure.MultiBand method_metadata_bands;
    BandStructure.IntBand method_MethodParameters_NB;
    BandStructure.CPRefBand method_MethodParameters_name_RUN;
    BandStructure.IntBand method_MethodParameters_flag_FH;
    BandStructure.MultiBand method_type_metadata_bands;
    BandStructure.MultiBand class_attr_bands;
    BandStructure.IntBand class_flags_hi;
    BandStructure.IntBand class_flags_lo;
    BandStructure.IntBand class_attr_count;
    BandStructure.IntBand class_attr_indexes;
    BandStructure.IntBand class_attr_calls;
    BandStructure.CPRefBand class_SourceFile_RUN;
    BandStructure.CPRefBand class_EnclosingMethod_RC;
    BandStructure.CPRefBand class_EnclosingMethod_RDN;
    BandStructure.CPRefBand class_Signature_RS;
    BandStructure.MultiBand class_metadata_bands;
    BandStructure.IntBand class_InnerClasses_N;
    BandStructure.CPRefBand class_InnerClasses_RC;
    BandStructure.IntBand class_InnerClasses_F;
    BandStructure.CPRefBand class_InnerClasses_outer_RCN;
    BandStructure.CPRefBand class_InnerClasses_name_RUN;
    BandStructure.IntBand class_ClassFile_version_minor_H;
    BandStructure.IntBand class_ClassFile_version_major_H;
    BandStructure.MultiBand class_type_metadata_bands;
    BandStructure.MultiBand code_bands;
    BandStructure.ByteBand code_headers;
    BandStructure.IntBand code_max_stack;
    BandStructure.IntBand code_max_na_locals;
    BandStructure.IntBand code_handler_count;
    BandStructure.IntBand code_handler_start_P;
    BandStructure.IntBand code_handler_end_PO;
    BandStructure.IntBand code_handler_catch_PO;
    BandStructure.CPRefBand code_handler_class_RCN;
    BandStructure.MultiBand code_attr_bands;
    BandStructure.IntBand code_flags_hi;
    BandStructure.IntBand code_flags_lo;
    BandStructure.IntBand code_attr_count;
    BandStructure.IntBand code_attr_indexes;
    BandStructure.IntBand code_attr_calls;
    BandStructure.MultiBand stackmap_bands;
    BandStructure.IntBand code_StackMapTable_N;
    BandStructure.IntBand code_StackMapTable_frame_T;
    BandStructure.IntBand code_StackMapTable_local_N;
    BandStructure.IntBand code_StackMapTable_stack_N;
    BandStructure.IntBand code_StackMapTable_offset;
    BandStructure.IntBand code_StackMapTable_T;
    BandStructure.CPRefBand code_StackMapTable_RC;
    BandStructure.IntBand code_StackMapTable_P;
    BandStructure.IntBand code_LineNumberTable_N;
    BandStructure.IntBand code_LineNumberTable_bci_P;
    BandStructure.IntBand code_LineNumberTable_line;
    BandStructure.IntBand code_LocalVariableTable_N;
    BandStructure.IntBand code_LocalVariableTable_bci_P;
    BandStructure.IntBand code_LocalVariableTable_span_O;
    BandStructure.CPRefBand code_LocalVariableTable_name_RU;
    BandStructure.CPRefBand code_LocalVariableTable_type_RS;
    BandStructure.IntBand code_LocalVariableTable_slot;
    BandStructure.IntBand code_LocalVariableTypeTable_N;
    BandStructure.IntBand code_LocalVariableTypeTable_bci_P;
    BandStructure.IntBand code_LocalVariableTypeTable_span_O;
    BandStructure.CPRefBand code_LocalVariableTypeTable_name_RU;
    BandStructure.CPRefBand code_LocalVariableTypeTable_type_RS;
    BandStructure.IntBand code_LocalVariableTypeTable_slot;
    BandStructure.MultiBand code_type_metadata_bands;
    BandStructure.MultiBand bc_bands;
    BandStructure.ByteBand bc_codes;
    BandStructure.IntBand bc_case_count;
    BandStructure.IntBand bc_case_value;
    BandStructure.ByteBand bc_byte;
    BandStructure.IntBand bc_short;
    BandStructure.IntBand bc_local;
    BandStructure.IntBand bc_label;
    BandStructure.CPRefBand bc_intref;
    BandStructure.CPRefBand bc_floatref;
    BandStructure.CPRefBand bc_longref;
    BandStructure.CPRefBand bc_doubleref;
    BandStructure.CPRefBand bc_stringref;
    BandStructure.CPRefBand bc_loadablevalueref;
    BandStructure.CPRefBand bc_classref;
    BandStructure.CPRefBand bc_fieldref;
    BandStructure.CPRefBand bc_methodref;
    BandStructure.CPRefBand bc_imethodref;
    BandStructure.CPRefBand bc_indyref;
    BandStructure.CPRefBand bc_thisfield;
    BandStructure.CPRefBand bc_superfield;
    BandStructure.CPRefBand bc_thismethod;
    BandStructure.CPRefBand bc_supermethod;
    BandStructure.IntBand bc_initref;
    BandStructure.CPRefBand bc_escref;
    BandStructure.IntBand bc_escrefsize;
    BandStructure.IntBand bc_escsize;
    BandStructure.ByteBand bc_escbyte;
    BandStructure.MultiBand file_bands;
    BandStructure.CPRefBand file_name;
    BandStructure.IntBand file_size_hi;
    BandStructure.IntBand file_size_lo;
    BandStructure.IntBand file_modtime;
    BandStructure.IntBand file_options;
    BandStructure.ByteBand file_bits;
    protected BandStructure.MultiBand[] metadataBands;
    protected BandStructure.MultiBand[] typeMetadataBands;
    public static final int ADH_CONTEXT_MASK = 3;
    public static final int ADH_BIT_SHIFT = 2;
    public static final int ADH_BIT_IS_LSB = 1;
    public static final int ATTR_INDEX_OVERFLOW = -1;
    public int[] attrIndexLimit;
    protected long[] attrFlagMask;
    protected long[] attrDefSeen;
    protected int[] attrOverflowMask;
    protected int attrClassFileVersionMask;
    protected Map<Attribute.Layout, BandStructure.Band[]> attrBandTable;
    protected final Attribute.Layout attrCodeEmpty;
    protected final Attribute.Layout attrInnerClassesEmpty;
    protected final Attribute.Layout attrClassFileVersion;
    protected final Attribute.Layout attrConstantValue;
    Map<Attribute.Layout, Integer> attrIndexTable;
    protected List<List<Attribute.Layout>> attrDefs;
    protected BandStructure.MultiBand[] attrBands;
    private static final int[][] shortCodeLimits;
    public final int shortCodeHeader_h_limit;
    static final int LONG_CODE_HEADER = 0;
    static int nextSeqForDebug;
    static File dumpDir;
    private Map<BandStructure.Band, BandStructure.Band> prevForAssertMap;
    static LinkedList<String> bandSequenceList;
    static final /* synthetic */ boolean $assertionsDisabled;
    
    protected abstract ConstantPool.Index getCPIndex(byte p0);
    
    public void initHighestClassVersion(Package.Version version) throws IOException {
        if (this.highestClassVersion != null) {
            throw new IOException("Highest class major version is already initialized to " + this.highestClassVersion + "; new setting is " + version);
        }
        this.highestClassVersion = version;
        this.adjustToClassVersion();
        return;
    }
    
    public Package.Version getHighestClassVersion() {
        return this.highestClassVersion;
    }
    
    protected BandStructure() {
        this.p200 = Utils.currentPropMap();
        this.verbose = this.p200.getInteger("com.sun.java.util.jar.pack.verbose");
        this.effort = this.p200.getInteger("pack.effort");
        if (this.effort == 0) {
            this.effort = 5;
        }
        this.optDumpBands = this.p200.getBoolean("com.sun.java.util.jar.pack.dump.bands");
        this.optDebugBands = this.p200.getBoolean("com.sun.java.util.jar.pack.debug.bands");
        this.optVaryCodings = !this.p200.getBoolean("com.sun.java.util.jar.pack.no.vary.codings");
        this.optBigStrings = !this.p200.getBoolean("com.sun.java.util.jar.pack.no.big.strings");
        this.highestClassVersion = null;
        this.isReader = (this instanceof PackageReader);
        this.allKQBands = new ArrayList();
        this.needPredefIndex = new ArrayList();
        this.all_bands = (BandStructure.MultiBand)new BandStructure.MultiBand("(package)", BandStructure.UNSIGNED5).init();
        this.archive_magic = this.all_bands.newByteBand("archive_magic");
        this.archive_header_0 = this.all_bands.newIntBand("archive_header_0", BandStructure.UNSIGNED5);
        this.archive_header_S = this.all_bands.newIntBand("archive_header_S", BandStructure.UNSIGNED5);
        this.archive_header_1 = this.all_bands.newIntBand("archive_header_1", BandStructure.UNSIGNED5);
        this.band_headers = this.all_bands.newByteBand("band_headers");
        this.cp_bands = this.all_bands.newMultiBand("(constant_pool)", BandStructure.DELTA5);
        this.cp_Utf8_prefix = this.cp_bands.newIntBand("cp_Utf8_prefix");
        this.cp_Utf8_suffix = this.cp_bands.newIntBand("cp_Utf8_suffix", BandStructure.UNSIGNED5);
        this.cp_Utf8_chars = this.cp_bands.newIntBand("cp_Utf8_chars", BandStructure.CHAR3);
        this.cp_Utf8_big_suffix = this.cp_bands.newIntBand("cp_Utf8_big_suffix");
        this.cp_Utf8_big_chars = this.cp_bands.newMultiBand("(cp_Utf8_big_chars)", BandStructure.DELTA5);
        this.cp_Int = this.cp_bands.newIntBand("cp_Int", BandStructure.UDELTA5);
        this.cp_Float = this.cp_bands.newIntBand("cp_Float", BandStructure.UDELTA5);
        this.cp_Long_hi = this.cp_bands.newIntBand("cp_Long_hi", BandStructure.UDELTA5);
        this.cp_Long_lo = this.cp_bands.newIntBand("cp_Long_lo");
        this.cp_Double_hi = this.cp_bands.newIntBand("cp_Double_hi", BandStructure.UDELTA5);
        this.cp_Double_lo = this.cp_bands.newIntBand("cp_Double_lo");
        this.cp_String = this.cp_bands.newCPRefBand("cp_String", BandStructure.UDELTA5, (byte)1);
        this.cp_Class = this.cp_bands.newCPRefBand("cp_Class", BandStructure.UDELTA5, (byte)1);
        this.cp_Signature_form = this.cp_bands.newCPRefBand("cp_Signature_form", (byte)1);
        this.cp_Signature_classes = this.cp_bands.newCPRefBand("cp_Signature_classes", BandStructure.UDELTA5, (byte)7);
        this.cp_Descr_name = this.cp_bands.newCPRefBand("cp_Descr_name", (byte)1);
        this.cp_Descr_type = this.cp_bands.newCPRefBand("cp_Descr_type", BandStructure.UDELTA5, (byte)13);
        this.cp_Field_class = this.cp_bands.newCPRefBand("cp_Field_class", (byte)7);
        this.cp_Field_desc = this.cp_bands.newCPRefBand("cp_Field_desc", BandStructure.UDELTA5, (byte)12);
        this.cp_Method_class = this.cp_bands.newCPRefBand("cp_Method_class", (byte)7);
        this.cp_Method_desc = this.cp_bands.newCPRefBand("cp_Method_desc", BandStructure.UDELTA5, (byte)12);
        this.cp_Imethod_class = this.cp_bands.newCPRefBand("cp_Imethod_class", (byte)7);
        this.cp_Imethod_desc = this.cp_bands.newCPRefBand("cp_Imethod_desc", BandStructure.UDELTA5, (byte)12);
        this.cp_MethodHandle_refkind = this.cp_bands.newIntBand("cp_MethodHandle_refkind", BandStructure.DELTA5);
        this.cp_MethodHandle_member = this.cp_bands.newCPRefBand("cp_MethodHandle_member", BandStructure.UDELTA5, (byte)52);
        this.cp_MethodType = this.cp_bands.newCPRefBand("cp_MethodType", BandStructure.UDELTA5, (byte)13);
        this.cp_BootstrapMethod_ref = this.cp_bands.newCPRefBand("cp_BootstrapMethod_ref", BandStructure.DELTA5, (byte)15);
        this.cp_BootstrapMethod_arg_count = this.cp_bands.newIntBand("cp_BootstrapMethod_arg_count", BandStructure.UDELTA5);
        this.cp_BootstrapMethod_arg = this.cp_bands.newCPRefBand("cp_BootstrapMethod_arg", BandStructure.DELTA5, (byte)51);
        this.cp_InvokeDynamic_spec = this.cp_bands.newCPRefBand("cp_InvokeDynamic_spec", BandStructure.DELTA5, (byte)17);
        this.cp_InvokeDynamic_desc = this.cp_bands.newCPRefBand("cp_InvokeDynamic_desc", BandStructure.UDELTA5, (byte)12);
        this.attr_definition_bands = this.all_bands.newMultiBand("(attr_definition_bands)", BandStructure.UNSIGNED5);
        this.attr_definition_headers = this.attr_definition_bands.newByteBand("attr_definition_headers");
        this.attr_definition_name = this.attr_definition_bands.newCPRefBand("attr_definition_name", (byte)1);
        this.attr_definition_layout = this.attr_definition_bands.newCPRefBand("attr_definition_layout", (byte)1);
        this.ic_bands = this.all_bands.newMultiBand("(ic_bands)", BandStructure.DELTA5);
        this.ic_this_class = this.ic_bands.newCPRefBand("ic_this_class", BandStructure.UDELTA5, (byte)7);
        this.ic_flags = this.ic_bands.newIntBand("ic_flags", BandStructure.UNSIGNED5);
        this.ic_outer_class = this.ic_bands.newCPRefBand("ic_outer_class", BandStructure.DELTA5, (byte)7, true);
        this.ic_name = this.ic_bands.newCPRefBand("ic_name", BandStructure.DELTA5, (byte)1, true);
        this.class_bands = this.all_bands.newMultiBand("(class_bands)", BandStructure.DELTA5);
        this.class_this = this.class_bands.newCPRefBand("class_this", (byte)7);
        this.class_super = this.class_bands.newCPRefBand("class_super", (byte)7);
        this.class_interface_count = this.class_bands.newIntBand("class_interface_count");
        this.class_interface = this.class_bands.newCPRefBand("class_interface", (byte)7);
        this.class_field_count = this.class_bands.newIntBand("class_field_count");
        this.class_method_count = this.class_bands.newIntBand("class_method_count");
        this.field_descr = this.class_bands.newCPRefBand("field_descr", (byte)12);
        this.field_attr_bands = this.class_bands.newMultiBand("(field_attr_bands)", BandStructure.UNSIGNED5);
        this.field_flags_hi = this.field_attr_bands.newIntBand("field_flags_hi");
        this.field_flags_lo = this.field_attr_bands.newIntBand("field_flags_lo");
        this.field_attr_count = this.field_attr_bands.newIntBand("field_attr_count");
        this.field_attr_indexes = this.field_attr_bands.newIntBand("field_attr_indexes");
        this.field_attr_calls = this.field_attr_bands.newIntBand("field_attr_calls");
        this.field_ConstantValue_KQ = this.field_attr_bands.newCPRefBand("field_ConstantValue_KQ", (byte)53);
        this.field_Signature_RS = this.field_attr_bands.newCPRefBand("field_Signature_RS", (byte)13);
        this.field_metadata_bands = this.field_attr_bands.newMultiBand("(field_metadata_bands)", BandStructure.UNSIGNED5);
        this.field_type_metadata_bands = this.field_attr_bands.newMultiBand("(field_type_metadata_bands)", BandStructure.UNSIGNED5);
        this.method_descr = this.class_bands.newCPRefBand("method_descr", BandStructure.MDELTA5, (byte)12);
        this.method_attr_bands = this.class_bands.newMultiBand("(method_attr_bands)", BandStructure.UNSIGNED5);
        this.method_flags_hi = this.method_attr_bands.newIntBand("method_flags_hi");
        this.method_flags_lo = this.method_attr_bands.newIntBand("method_flags_lo");
        this.method_attr_count = this.method_attr_bands.newIntBand("method_attr_count");
        this.method_attr_indexes = this.method_attr_bands.newIntBand("method_attr_indexes");
        this.method_attr_calls = this.method_attr_bands.newIntBand("method_attr_calls");
        this.method_Exceptions_N = this.method_attr_bands.newIntBand("method_Exceptions_N");
        this.method_Exceptions_RC = this.method_attr_bands.newCPRefBand("method_Exceptions_RC", (byte)7);
        this.method_Signature_RS = this.method_attr_bands.newCPRefBand("method_Signature_RS", (byte)13);
        this.method_metadata_bands = this.method_attr_bands.newMultiBand("(method_metadata_bands)", BandStructure.UNSIGNED5);
        this.method_MethodParameters_NB = this.method_attr_bands.newIntBand("method_MethodParameters_NB", BandStructure.BYTE1);
        this.method_MethodParameters_name_RUN = this.method_attr_bands.newCPRefBand("method_MethodParameters_name_RUN", BandStructure.UNSIGNED5, (byte)1, true);
        this.method_MethodParameters_flag_FH = this.method_attr_bands.newIntBand("method_MethodParameters_flag_FH");
        this.method_type_metadata_bands = this.method_attr_bands.newMultiBand("(method_type_metadata_bands)", BandStructure.UNSIGNED5);
        this.class_attr_bands = this.class_bands.newMultiBand("(class_attr_bands)", BandStructure.UNSIGNED5);
        this.class_flags_hi = this.class_attr_bands.newIntBand("class_flags_hi");
        this.class_flags_lo = this.class_attr_bands.newIntBand("class_flags_lo");
        this.class_attr_count = this.class_attr_bands.newIntBand("class_attr_count");
        this.class_attr_indexes = this.class_attr_bands.newIntBand("class_attr_indexes");
        this.class_attr_calls = this.class_attr_bands.newIntBand("class_attr_calls");
        this.class_SourceFile_RUN = this.class_attr_bands.newCPRefBand("class_SourceFile_RUN", BandStructure.UNSIGNED5, (byte)1, true);
        this.class_EnclosingMethod_RC = this.class_attr_bands.newCPRefBand("class_EnclosingMethod_RC", (byte)7);
        this.class_EnclosingMethod_RDN = this.class_attr_bands.newCPRefBand("class_EnclosingMethod_RDN", BandStructure.UNSIGNED5, (byte)12, true);
        this.class_Signature_RS = this.class_attr_bands.newCPRefBand("class_Signature_RS", (byte)13);
        this.class_metadata_bands = this.class_attr_bands.newMultiBand("(class_metadata_bands)", BandStructure.UNSIGNED5);
        this.class_InnerClasses_N = this.class_attr_bands.newIntBand("class_InnerClasses_N");
        this.class_InnerClasses_RC = this.class_attr_bands.newCPRefBand("class_InnerClasses_RC", (byte)7);
        this.class_InnerClasses_F = this.class_attr_bands.newIntBand("class_InnerClasses_F");
        this.class_InnerClasses_outer_RCN = this.class_attr_bands.newCPRefBand("class_InnerClasses_outer_RCN", BandStructure.UNSIGNED5, (byte)7, true);
        this.class_InnerClasses_name_RUN = this.class_attr_bands.newCPRefBand("class_InnerClasses_name_RUN", BandStructure.UNSIGNED5, (byte)1, true);
        this.class_ClassFile_version_minor_H = this.class_attr_bands.newIntBand("class_ClassFile_version_minor_H");
        this.class_ClassFile_version_major_H = this.class_attr_bands.newIntBand("class_ClassFile_version_major_H");
        this.class_type_metadata_bands = this.class_attr_bands.newMultiBand("(class_type_metadata_bands)", BandStructure.UNSIGNED5);
        this.code_bands = this.class_bands.newMultiBand("(code_bands)", BandStructure.UNSIGNED5);
        this.code_headers = this.code_bands.newByteBand("code_headers");
        this.code_max_stack = this.code_bands.newIntBand("code_max_stack", BandStructure.UNSIGNED5);
        this.code_max_na_locals = this.code_bands.newIntBand("code_max_na_locals", BandStructure.UNSIGNED5);
        this.code_handler_count = this.code_bands.newIntBand("code_handler_count", BandStructure.UNSIGNED5);
        this.code_handler_start_P = this.code_bands.newIntBand("code_handler_start_P", BandStructure.BCI5);
        this.code_handler_end_PO = this.code_bands.newIntBand("code_handler_end_PO", BandStructure.BRANCH5);
        this.code_handler_catch_PO = this.code_bands.newIntBand("code_handler_catch_PO", BandStructure.BRANCH5);
        this.code_handler_class_RCN = this.code_bands.newCPRefBand("code_handler_class_RCN", BandStructure.UNSIGNED5, (byte)7, true);
        this.code_attr_bands = this.class_bands.newMultiBand("(code_attr_bands)", BandStructure.UNSIGNED5);
        this.code_flags_hi = this.code_attr_bands.newIntBand("code_flags_hi");
        this.code_flags_lo = this.code_attr_bands.newIntBand("code_flags_lo");
        this.code_attr_count = this.code_attr_bands.newIntBand("code_attr_count");
        this.code_attr_indexes = this.code_attr_bands.newIntBand("code_attr_indexes");
        this.code_attr_calls = this.code_attr_bands.newIntBand("code_attr_calls");
        this.stackmap_bands = this.code_attr_bands.newMultiBand("(StackMapTable_bands)", BandStructure.UNSIGNED5);
        this.code_StackMapTable_N = this.stackmap_bands.newIntBand("code_StackMapTable_N");
        this.code_StackMapTable_frame_T = this.stackmap_bands.newIntBand("code_StackMapTable_frame_T", BandStructure.BYTE1);
        this.code_StackMapTable_local_N = this.stackmap_bands.newIntBand("code_StackMapTable_local_N");
        this.code_StackMapTable_stack_N = this.stackmap_bands.newIntBand("code_StackMapTable_stack_N");
        this.code_StackMapTable_offset = this.stackmap_bands.newIntBand("code_StackMapTable_offset", BandStructure.UNSIGNED5);
        this.code_StackMapTable_T = this.stackmap_bands.newIntBand("code_StackMapTable_T", BandStructure.BYTE1);
        this.code_StackMapTable_RC = this.stackmap_bands.newCPRefBand("code_StackMapTable_RC", (byte)7);
        this.code_StackMapTable_P = this.stackmap_bands.newIntBand("code_StackMapTable_P", BandStructure.BCI5);
        this.code_LineNumberTable_N = this.code_attr_bands.newIntBand("code_LineNumberTable_N");
        this.code_LineNumberTable_bci_P = this.code_attr_bands.newIntBand("code_LineNumberTable_bci_P", BandStructure.BCI5);
        this.code_LineNumberTable_line = this.code_attr_bands.newIntBand("code_LineNumberTable_line");
        this.code_LocalVariableTable_N = this.code_attr_bands.newIntBand("code_LocalVariableTable_N");
        this.code_LocalVariableTable_bci_P = this.code_attr_bands.newIntBand("code_LocalVariableTable_bci_P", BandStructure.BCI5);
        this.code_LocalVariableTable_span_O = this.code_attr_bands.newIntBand("code_LocalVariableTable_span_O", BandStructure.BRANCH5);
        this.code_LocalVariableTable_name_RU = this.code_attr_bands.newCPRefBand("code_LocalVariableTable_name_RU", (byte)1);
        this.code_LocalVariableTable_type_RS = this.code_attr_bands.newCPRefBand("code_LocalVariableTable_type_RS", (byte)13);
        this.code_LocalVariableTable_slot = this.code_attr_bands.newIntBand("code_LocalVariableTable_slot");
        this.code_LocalVariableTypeTable_N = this.code_attr_bands.newIntBand("code_LocalVariableTypeTable_N");
        this.code_LocalVariableTypeTable_bci_P = this.code_attr_bands.newIntBand("code_LocalVariableTypeTable_bci_P", BandStructure.BCI5);
        this.code_LocalVariableTypeTable_span_O = this.code_attr_bands.newIntBand("code_LocalVariableTypeTable_span_O", BandStructure.BRANCH5);
        this.code_LocalVariableTypeTable_name_RU = this.code_attr_bands.newCPRefBand("code_LocalVariableTypeTable_name_RU", (byte)1);
        this.code_LocalVariableTypeTable_type_RS = this.code_attr_bands.newCPRefBand("code_LocalVariableTypeTable_type_RS", (byte)13);
        this.code_LocalVariableTypeTable_slot = this.code_attr_bands.newIntBand("code_LocalVariableTypeTable_slot");
        this.code_type_metadata_bands = this.code_attr_bands.newMultiBand("(code_type_metadata_bands)", BandStructure.UNSIGNED5);
        this.bc_bands = this.all_bands.newMultiBand("(byte_codes)", BandStructure.UNSIGNED5);
        this.bc_codes = this.bc_bands.newByteBand("bc_codes");
        this.bc_case_count = this.bc_bands.newIntBand("bc_case_count");
        this.bc_case_value = this.bc_bands.newIntBand("bc_case_value", BandStructure.DELTA5);
        this.bc_byte = this.bc_bands.newByteBand("bc_byte");
        this.bc_short = this.bc_bands.newIntBand("bc_short", BandStructure.DELTA5);
        this.bc_local = this.bc_bands.newIntBand("bc_local");
        this.bc_label = this.bc_bands.newIntBand("bc_label", BandStructure.BRANCH5);
        this.bc_intref = this.bc_bands.newCPRefBand("bc_intref", BandStructure.DELTA5, (byte)3);
        this.bc_floatref = this.bc_bands.newCPRefBand("bc_floatref", BandStructure.DELTA5, (byte)4);
        this.bc_longref = this.bc_bands.newCPRefBand("bc_longref", BandStructure.DELTA5, (byte)5);
        this.bc_doubleref = this.bc_bands.newCPRefBand("bc_doubleref", BandStructure.DELTA5, (byte)6);
        this.bc_stringref = this.bc_bands.newCPRefBand("bc_stringref", BandStructure.DELTA5, (byte)8);
        this.bc_loadablevalueref = this.bc_bands.newCPRefBand("bc_loadablevalueref", BandStructure.DELTA5, (byte)51);
        this.bc_classref = this.bc_bands.newCPRefBand("bc_classref", BandStructure.UNSIGNED5, (byte)7, true);
        this.bc_fieldref = this.bc_bands.newCPRefBand("bc_fieldref", BandStructure.DELTA5, (byte)9);
        this.bc_methodref = this.bc_bands.newCPRefBand("bc_methodref", (byte)10);
        this.bc_imethodref = this.bc_bands.newCPRefBand("bc_imethodref", BandStructure.DELTA5, (byte)11);
        this.bc_indyref = this.bc_bands.newCPRefBand("bc_indyref", BandStructure.DELTA5, (byte)18);
        this.bc_thisfield = this.bc_bands.newCPRefBand("bc_thisfield", (byte)0);
        this.bc_superfield = this.bc_bands.newCPRefBand("bc_superfield", (byte)0);
        this.bc_thismethod = this.bc_bands.newCPRefBand("bc_thismethod", (byte)0);
        this.bc_supermethod = this.bc_bands.newCPRefBand("bc_supermethod", (byte)0);
        this.bc_initref = this.bc_bands.newIntBand("bc_initref");
        this.bc_escref = this.bc_bands.newCPRefBand("bc_escref", (byte)50);
        this.bc_escrefsize = this.bc_bands.newIntBand("bc_escrefsize");
        this.bc_escsize = this.bc_bands.newIntBand("bc_escsize");
        this.bc_escbyte = this.bc_bands.newByteBand("bc_escbyte");
        this.file_bands = this.all_bands.newMultiBand("(file_bands)", BandStructure.UNSIGNED5);
        this.file_name = this.file_bands.newCPRefBand("file_name", (byte)1);
        this.file_size_hi = this.file_bands.newIntBand("file_size_hi");
        this.file_size_lo = this.file_bands.newIntBand("file_size_lo");
        this.file_modtime = this.file_bands.newIntBand("file_modtime", BandStructure.DELTA5);
        this.file_options = this.file_bands.newIntBand("file_options");
        this.file_bits = this.file_bands.newByteBand("file_bits");
        (this.metadataBands = new BandStructure.MultiBand[4])[0] = this.class_metadata_bands;
        this.metadataBands[1] = this.field_metadata_bands;
        this.metadataBands[2] = this.method_metadata_bands;
        (this.typeMetadataBands = new BandStructure.MultiBand[4])[0] = this.class_type_metadata_bands;
        this.typeMetadataBands[1] = this.field_type_metadata_bands;
        this.typeMetadataBands[2] = this.method_type_metadata_bands;
        this.typeMetadataBands[3] = this.code_type_metadata_bands;
        this.attrIndexLimit = new int[4];
        this.attrFlagMask = new long[4];
        this.attrDefSeen = new long[4];
        this.attrOverflowMask = new int[4];
        this.attrBandTable = new HashMap();
        this.attrIndexTable = new HashMap();
        this.attrDefs = new FixedList(4);
        int i = 0;
        while (i < 4) {
            assert this.attrIndexLimit[i] == 0;
            this.attrIndexLimit[i] = 32;
            this.attrDefs.set(i, new ArrayList(Collections.nCopies(this.attrIndexLimit[i], (T)null)));
            ++i;
        }
        this.attrInnerClassesEmpty = this.predefineAttribute(23, 0, null, "InnerClasses", "");
        assert this.attrInnerClassesEmpty == Package.attrInnerClassesEmpty;
        this.predefineAttribute(17, 0, new BandStructure.Band[] { this.class_SourceFile_RUN }, "SourceFile", "RUNH");
        this.predefineAttribute(18, 0, new BandStructure.Band[] { this.class_EnclosingMethod_RC, this.class_EnclosingMethod_RDN }, "EnclosingMethod", "RCHRDNH");
        this.attrClassFileVersion = this.predefineAttribute(24, 0, new BandStructure.Band[] { this.class_ClassFile_version_minor_H, this.class_ClassFile_version_major_H }, ".ClassFile.version", "HH");
        this.predefineAttribute(19, 0, new BandStructure.Band[] { this.class_Signature_RS }, "Signature", "RSH");
        this.predefineAttribute(20, 0, null, "Deprecated", "");
        this.predefineAttribute(16, 0, null, ".Overflow", "");
        this.attrConstantValue = this.predefineAttribute(17, 1, new BandStructure.Band[] { this.field_ConstantValue_KQ }, "ConstantValue", "KQH");
        this.predefineAttribute(19, 1, new BandStructure.Band[] { this.field_Signature_RS }, "Signature", "RSH");
        this.predefineAttribute(20, 1, null, "Deprecated", "");
        this.predefineAttribute(16, 1, null, ".Overflow", "");
        this.attrCodeEmpty = this.predefineAttribute(17, 2, null, "Code", "");
        this.predefineAttribute(18, 2, new BandStructure.Band[] { this.method_Exceptions_N, this.method_Exceptions_RC }, "Exceptions", "NH[RCH]");
        this.predefineAttribute(26, 2, new BandStructure.Band[] { this.method_MethodParameters_NB, this.method_MethodParameters_name_RUN, this.method_MethodParameters_flag_FH }, "MethodParameters", "NB[RUNHFH]");
        assert this.attrCodeEmpty == Package.attrCodeEmpty;
        this.predefineAttribute(19, 2, new BandStructure.Band[] { this.method_Signature_RS }, "Signature", "RSH");
        this.predefineAttribute(20, 2, null, "Deprecated", "");
        this.predefineAttribute(16, 2, null, ".Overflow", "");
        for (int j = 0; j < 4; ++j) {
            BandStructure.MultiBand multiBand = this.metadataBands[j];
            if (j != 3) {
                this.predefineAttribute(21, Constants.ATTR_CONTEXT_NAME[j] + "_RVA_", multiBand, Attribute.lookup(null, j, "RuntimeVisibleAnnotations"));
                this.predefineAttribute(22, Constants.ATTR_CONTEXT_NAME[j] + "_RIA_", multiBand, Attribute.lookup(null, j, "RuntimeInvisibleAnnotations"));
                if (j == 2) {
                    this.predefineAttribute(23, "method_RVPA_", multiBand, Attribute.lookup(null, j, "RuntimeVisibleParameterAnnotations"));
                    this.predefineAttribute(24, "method_RIPA_", multiBand, Attribute.lookup(null, j, "RuntimeInvisibleParameterAnnotations"));
                    this.predefineAttribute(25, "method_AD_", multiBand, Attribute.lookup(null, j, "AnnotationDefault"));
                }
            }
            BandStructure.MultiBand multiBand2 = this.typeMetadataBands[j];
            this.predefineAttribute(27, Constants.ATTR_CONTEXT_NAME[j] + "_RVTA_", multiBand2, Attribute.lookup(null, j, "RuntimeVisibleTypeAnnotations"));
            this.predefineAttribute(28, Constants.ATTR_CONTEXT_NAME[j] + "_RITA_", multiBand2, Attribute.lookup(null, j, "RuntimeInvisibleTypeAnnotations"));
        }
        Attribute.Layout layout = Attribute.lookup(null, 3, "StackMapTable").layout();
        this.predefineAttribute(0, 3, this.stackmap_bands.toArray(), layout.name(), layout.layout());
        this.predefineAttribute(1, 3, new BandStructure.Band[] { this.code_LineNumberTable_N, this.code_LineNumberTable_bci_P, this.code_LineNumberTable_line }, "LineNumberTable", "NH[PHH]");
        this.predefineAttribute(2, 3, new BandStructure.Band[] { this.code_LocalVariableTable_N, this.code_LocalVariableTable_bci_P, this.code_LocalVariableTable_span_O, this.code_LocalVariableTable_name_RU, this.code_LocalVariableTable_type_RS, this.code_LocalVariableTable_slot }, "LocalVariableTable", "NH[PHOHRUHRSHH]");
        this.predefineAttribute(3, 3, new BandStructure.Band[] { this.code_LocalVariableTypeTable_N, this.code_LocalVariableTypeTable_bci_P, this.code_LocalVariableTypeTable_span_O, this.code_LocalVariableTypeTable_name_RU, this.code_LocalVariableTypeTable_type_RS, this.code_LocalVariableTypeTable_slot }, "LocalVariableTypeTable", "NH[PHOHRUHRSHH]");
        this.predefineAttribute(16, 3, null, ".Overflow", "");
        for (int k = 0; k < 4; ++k) {
            this.attrDefSeen[k] = 0L;
        }
        for (int l = 0; l < 4; ++l) {
            this.attrOverflowMask[l] = 65536;
            this.attrIndexLimit[l] = 0;
        }
        this.attrClassFileVersionMask = 16777216;
        (this.attrBands = new BandStructure.MultiBand[4])[0] = this.class_attr_bands;
        this.attrBands[1] = this.field_attr_bands;
        this.attrBands[2] = this.method_attr_bands;
        this.attrBands[3] = this.code_attr_bands;
        this.shortCodeHeader_h_limit = BandStructure.shortCodeLimits.length;
        return;
    }
    
    public static Coding codingForIndex(int n) {
        return (n < BandStructure.basicCodings.length) ? BandStructure.basicCodings[n] : null;
    }
    
    public static int indexOf(Coding coding) {
        Integer n = (Integer)BandStructure.basicCodingIndexes.get(coding);
        if (n == null) {
            return 0;
        }
        return (int)n;
    }
    
    public static Coding[] getBasicCodings() {
        return (Coding[])BandStructure.basicCodings.clone();
    }
    
    protected CodingMethod getBandHeader(int n, Coding coding) {
        CodingMethod[] array = { null };
        this.bandHeaderBytes[--this.bandHeaderBytePos] = (byte)n;
        this.bandHeaderBytePos0 = this.bandHeaderBytePos;
        this.bandHeaderBytePos = parseMetaCoding(this.bandHeaderBytes, this.bandHeaderBytePos, coding, array);
        return array[0];
    }
    
    public static int parseMetaCoding(byte[] array, int n, Coding coding, CodingMethod[] array2) {
        if ((array[n] & 0xFF) == 0x0) {
            array2[0] = coding;
            return n + 1;
        }
        int metaCoding = Coding.parseMetaCoding(array, n, coding, array2);
        if (metaCoding > n) {
            return metaCoding;
        }
        int metaCoding2 = PopulationCoding.parseMetaCoding(array, n, coding, array2);
        if (metaCoding2 > n) {
            return metaCoding2;
        }
        int metaCoding3 = AdaptiveCoding.parseMetaCoding(array, n, coding, array2);
        if (metaCoding3 > n) {
            return metaCoding3;
        }
        throw new RuntimeException("Bad meta-coding op " + (array[n] & 0xFF));
    }
    
    static boolean phaseIsRead(int n) {
        return n % 2 == 0;
    }
    
    static int phaseCmp(int n, int n2) {
        assert n2 % 8 == 0;
        return n - n2;
    }
    
    static int getIntTotal(int[] array) {
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            n += array[i];
        }
        return n;
    }
    
    int encodeRef(ConstantPool.Entry obj, ConstantPool.Index index) {
        if (index == null) {
            throw new RuntimeException("null index for " + obj.stringValue());
        }
        int index2 = index.indexOf(obj);
        if (this.verbose > 2) {
            Utils.log.fine("putRef " + index2 + " => " + obj);
        }
        return index2;
    }
    
    ConstantPool.Entry decodeRef(int n, ConstantPool.Index obj) {
        if (n < 0 || n >= obj.size()) {
            Utils.log.warning("decoding bad ref " + n + " in " + obj);
        }
        ConstantPool.Entry entry = obj.getEntry(n);
        if (this.verbose > 2) {
            Utils.log.fine("getRef " + n + " => " + entry);
            return entry;
        }
        return entry;
    }
    
    protected CodingChooser getCodingChooser() {
        if (this.codingChooser == null) {
            this.codingChooser = new CodingChooser(this.effort, BandStructure.basicCodings);
            if (this.codingChooser.stress != null && this instanceof PackageWriter) {
                ArrayList<Object> classes = ((PackageWriter)this).pkg.classes;
                if (!classes.isEmpty()) {
                    this.codingChooser.addStressSeed(((Package.Class)classes.get(0)).getName().hashCode());
                }
            }
        }
        return this.codingChooser;
    }
    
    public CodingMethod chooseCoding(int[] array, int n, int n2, Coding coding, String str, int[] array2) {
        assert !(!this.optVaryCodings);
        if (this.effort <= 1) {
            return coding;
        }
        CodingChooser codingChooser = this.getCodingChooser();
        if (this.verbose > 1 || codingChooser.verbose > 1) {
            Utils.log.fine("--- chooseCoding " + str);
        }
        return codingChooser.choose(array, n, n2, coding, array2);
    }
    
    protected static int decodeEscapeValue(int n, Coding coding) {
        if (coding.B() == 1 || coding.L() == 0) {
            return -1;
        }
        if (coding.S() != 0) {
            if (-256 <= n && n <= -1 && coding.min() <= -256) {
                int n3 = -1 - n;
                assert n3 >= 0 && n3 < 256;
                return n3;
            }
        }
        else {
            int l = coding.L();
            if (l <= n && n <= l + 255 && coding.max() >= l + 255) {
                int n2 = n - l;
                assert n2 >= 0 && n2 < 256;
                return n2;
            }
        }
        return -1;
    }
    
    protected static int encodeEscapeValue(int i, Coding obj) {
        assert i >= 0 && i < 256;
        assert obj.B() > 1 && obj.L() > 0;
        int j;
        if (obj.S() != 0) {
            assert obj.min() <= -256;
            j = -1 - i;
        }
        else {
            int l = obj.L();
            assert obj.max() >= l + 255;
            j = i + l;
        }
        assert decodeEscapeValue(j, obj) == i : obj + " XB=" + i + " X=" + j;
        return j;
    }
    
    void writeAllBandsTo(OutputStream outputStream) throws IOException {
        this.outputCounter = new ByteCounter(outputStream);
        this.all_bands.writeTo(this.outputCounter);
        if (this.verbose > 0) {
            long count = this.outputCounter.getCount();
            Utils.log.info("Wrote total of " + count + " bytes.");
            assert count == this.archiveSize0 + this.archiveSize1;
        }
        this.outputCounter = null;
    }
    
    static BandStructure.IntBand getAttrBand(BandStructure.MultiBand multiBand, int n) {
        BandStructure.IntBand intBand = (BandStructure.IntBand)multiBand.get(n);
        switch (n) {
            case 0: {
                assert !(!intBand.name().endsWith("_flags_hi"));
                break;
            }
            case 1: {
                assert !(!intBand.name().endsWith("_flags_lo"));
                break;
            }
            case 2: {
                assert !(!intBand.name().endsWith("_attr_count"));
                break;
            }
            case 3: {
                assert !(!intBand.name().endsWith("_attr_indexes"));
                break;
            }
            case 4: {
                assert !(!intBand.name().endsWith("_attr_calls"));
                break;
            }
            default: {
                assert false;
                break;
            }
        }
        return intBand;
    }
    
    protected void setBandIndexes() {
        for (Object[] array : this.needPredefIndex) {
            ((BandStructure.CPRefBand)array[0]).setIndex(this.getCPIndex((byte)array[1]));
        }
        this.needPredefIndex = null;
        if (this.verbose > 3) {
            printCDecl(this.all_bands);
        }
    }
    
    protected void setBandIndex(BandStructure.CPRefBand cpRefBand, byte b) {
        Object[] array = { cpRefBand, Byte.valueOf(b) };
        if (b == 53) {
            this.allKQBands.add(cpRefBand);
        }
        else if (this.needPredefIndex != null) {
            this.needPredefIndex.add(array);
        }
        else {
            cpRefBand.setIndex(this.getCPIndex(b));
        }
    }
    
    protected void setConstantValueIndex(Package.Class.Field obj) {
        ConstantPool.Index cpIndex = null;
        if (obj != null) {
            int literalTag = obj.getLiteralTag();
            cpIndex = this.getCPIndex((byte)literalTag);
            if (this.verbose > 2) {
                Utils.log.fine("setConstantValueIndex " + obj + " " + ConstantPool.tagName(literalTag) + " => " + cpIndex);
            }
            assert cpIndex != null;
        }
        Iterator<BandStructure.CPRefBand> iterator = this.allKQBands.iterator();
        while (iterator.hasNext()) {
            ((BandStructure.CPRefBand)iterator.next()).setIndex(cpIndex);
        }
    }
    
    private void adjustToClassVersion() throws IOException {
        if (this.getHighestClassVersion().lessThan(Constants.JAVA6_MAX_CLASS_VERSION)) {
            if (this.verbose > 0) {
                Utils.log.fine("Legacy package version");
            }
            this.undefineAttribute(0, 3);
        }
    }
    
    protected void initAttrIndexLimit() {
        int i = 0;
        while (i < 4) {
            assert this.attrIndexLimit[i] == 0;
            this.attrIndexLimit[i] = (this.haveFlagsHi(i) ? 63 : 32);
            List<Object> list = (List)this.attrDefs.get(i);
            assert list.size() == 32;
            list.addAll(Collections.nCopies(this.attrIndexLimit[i] - list.size(), (T)null));
            ++i;
        }
    }
    
    protected boolean haveFlagsHi(int n) {
        int n2 = 1 << 9 + n;
        switch (n) {
            case 0: {
                assert n2 == 512;
                break;
            }
            case 1: {
                assert n2 == 1024;
                break;
            }
            case 2: {
                assert n2 == 2048;
                break;
            }
            case 3: {
                assert n2 == 4096;
                break;
            }
            default: {
                assert false;
                break;
            }
        }
        return testBit(this.archiveOptions, n2);
    }
    
    protected List<Attribute.Layout> getPredefinedAttrs(int n) {
        assert this.attrIndexLimit[n] != 0;
        ArrayList<Attribute.Layout> list = new ArrayList(this.attrIndexLimit[n]);
        for (int i = 0; i < this.attrIndexLimit[n]; ++i) {
            if (!testBit(this.attrDefSeen[n], 1L << i)) {
                Attribute.Layout layout = (Attribute.Layout)((List)this.attrDefs.get(n)).get(i);
                if (layout != null) {
                    assert !(!this.isPredefinedAttr(n, i));
                    list.add(layout);
                }
            }
        }
        return list;
    }
    
    protected boolean isPredefinedAttr(int n, int n2) {
        assert this.attrIndexLimit[n] != 0;
        return n2 < this.attrIndexLimit[n] && !testBit(this.attrDefSeen[n], 1L << n2) && ((List)this.attrDefs.get(n)).get(n2) != null;
    }
    
    protected void adjustSpecialAttrMasks() {
        this.attrClassFileVersionMask = (int)((long)this.attrClassFileVersionMask & ~this.attrDefSeen[0]);
        for (int i = 0; i < 4; ++i) {
            int[] attrOverflowMask = this.attrOverflowMask;
            int n = i;
            attrOverflowMask[n] = (int)((long)attrOverflowMask[n] & ~this.attrDefSeen[i]);
        }
    }
    
    protected Attribute makeClassFileVersionAttr(Package.Version version) {
        return this.attrClassFileVersion.addContent(version.asBytes());
    }
    
    protected Package.Version parseClassFileVersionAttr(Attribute attribute) {
        assert attribute.layout() == this.attrClassFileVersion;
        assert attribute.size() == 4;
        return Package.Version.of(attribute.bytes());
    }
    
    private boolean assertBandOKForElems(BandStructure.Band[] array, Attribute.Layout.Element[] array2) {
        int i = 0;
        while (i < array2.length) {
            assert !(!this.assertBandOKForElem(array, array2[i]));
            ++i;
        }
        return true;
    }
    
    private boolean assertBandOKForElem(BandStructure.Band[] array, Attribute.Layout.Element obj) {
        BandStructure.Band obj2 = null;
        if (obj.bandIndex != -1) {
            obj2 = array[obj.bandIndex];
        }
        Coding coding = BandStructure.UNSIGNED5;
        boolean n = true;
        switch (obj.kind) {
            case 1: {
                if (obj.flagTest((byte)1)) {
                    coding = BandStructure.SIGNED5;
                    break;
                }
                if (obj.len == 1) {
                    coding = BandStructure.BYTE1;
                    break;
                }
                break;
            }
            case 2: {
                if (!obj.flagTest((byte)2)) {
                    coding = BandStructure.BCI5;
                    break;
                }
                coding = BandStructure.BRANCH5;
                break;
            }
            case 3: {
                coding = BandStructure.BRANCH5;
                break;
            }
            case 4: {
                if (obj.len == 1) {
                    coding = BandStructure.BYTE1;
                    break;
                }
                break;
            }
            case 5: {
                if (obj.len == 1) {
                    coding = BandStructure.BYTE1;
                }
                this.assertBandOKForElems(array, obj.body);
                break;
            }
            case 7: {
                if (obj.flagTest((byte)1)) {
                    coding = BandStructure.SIGNED5;
                }
                else if (obj.len == 1) {
                    coding = BandStructure.BYTE1;
                }
                this.assertBandOKForElems(array, obj.body);
                break;
            }
            case 8: {
                assert obj2 == null;
                this.assertBandOKForElems(array, obj.body);
                return true;
            }
            case 9: {
                assert obj2 == null;
                return true;
            }
            case 10: {
                assert obj2 == null;
                this.assertBandOKForElems(array, obj.body);
                return true;
            }
            case 6: {
                n = false;
                assert !(!(obj2 instanceof BandStructure.CPRefBand));
                assert ((BandStructure.CPRefBand)obj2).nullOK == obj.flagTest((byte)4);
                break;
            }
            default: {
                assert false;
                break;
            }
        }
        assert obj2.regularCoding == coding : obj + " // " + obj2;
        if (n && !BandStructure.$assertionsDisabled && !(obj2 instanceof BandStructure.IntBand)) {
            throw new AssertionError();
        }
        return true;
    }
    
    private Attribute.Layout predefineAttribute(int n, int n2, BandStructure.Band[] a, String s, String s2) {
        Attribute.Layout layout = Attribute.find(n2, s, s2).layout();
        if (n >= 0) {
            this.setAttributeLayoutIndex(layout, n);
        }
        if (a == null) {
            a = new BandStructure.Band[0];
        }
        assert this.attrBandTable.get(layout) == null;
        this.attrBandTable.put(layout, a);
        assert layout.bandCount == a.length : layout + " // " + Arrays.asList(a);
        assert !(!this.assertBandOKForElems(a, layout.elems));
        return layout;
    }
    
    private Attribute.Layout predefineAttribute(int n, String s, BandStructure.MultiBand multiBand, Attribute attribute) {
        Attribute.Layout layout = attribute.layout();
        return this.predefineAttribute(n, layout.ctype(), this.makeNewAttributeBands(s, layout, multiBand), layout.name(), layout.layout());
    }
    
    private void undefineAttribute(int i, int n) {
        if (this.verbose > 1) {
            System.out.println("Removing predefined " + Constants.ATTR_CONTEXT_NAME[n] + " attribute on bit " + i);
        }
        List<Object> list = (List)this.attrDefs.get(n);
        Attribute.Layout layout = (Attribute.Layout)list.get(i);
        assert layout != null;
        list.set(i, null);
        this.attrIndexTable.put(layout, null);
        assert i < 64;
        long[] attrDefSeen = this.attrDefSeen;
        attrDefSeen[n] &= ~(1L << i);
        long[] attrFlagMask = this.attrFlagMask;
        attrFlagMask[n] &= ~(1L << i);
        BandStructure.Band[] array = (BandStructure.Band[])this.attrBandTable.get(layout);
        for (int j = 0; j < array.length; ++j) {
            array[j].doneWithUnusedBand();
        }
        return;
    }
    
    void makeNewAttributeBands() {
        this.adjustSpecialAttrMasks();
        int i = 0;
        while (i < 4) {
            String str = Constants.ATTR_CONTEXT_NAME[i];
            BandStructure.MultiBand multiBand = this.attrBands[i];
            long n = this.attrDefSeen[i];
            assert (n & ~this.attrFlagMask[i]) == 0x0L;
            for (int j = 0; j < ((List)this.attrDefs.get(i)).size(); ++j) {
                Attribute.Layout obj = (Attribute.Layout)((List)this.attrDefs.get(i)).get(j);
                if (obj != null && obj.bandCount != 0) {
                    if (j < this.attrIndexLimit[i] && !testBit(n, 1L << j)) {
                        assert this.attrBandTable.get(obj) != null;
                    }
                    else {
                        multiBand.size();
                        String string = str + "_" + obj.name() + "_";
                        if (this.verbose > 1) {
                            Utils.log.fine("Making new bands for " + obj);
                        }
                        BandStructure.Band[] newAttributeBands = this.makeNewAttributeBands(string, obj, multiBand);
                        assert newAttributeBands.length == obj.bandCount;
                        BandStructure.Band[] array = (BandStructure.Band[])this.attrBandTable.put(obj, newAttributeBands);
                        if (array != null) {
                            for (int k = 0; k < array.length; ++k) {
                                array[k].doneWithUnusedBand();
                            }
                        }
                    }
                }
            }
            ++i;
        }
    }
    
    private BandStructure.Band[] makeNewAttributeBands(String s, Attribute.Layout layout, BandStructure.MultiBand multiBand) {
        int size = multiBand.size();
        this.makeNewAttributeBands(s, layout.elems, multiBand);
        int n = multiBand.size() - size;
        BandStructure.Band[] array = new BandStructure.Band[n];
        for (int i = 0; i < n; ++i) {
            array[i] = multiBand.get(size + i);
        }
        return array;
    }
    
    private void makeNewAttributeBands(String str, Attribute.Layout.Element[] array, BandStructure.MultiBand multiBand) {
        for (int i = 0; i < array.length; ++i) {
            Attribute.Layout.Element element = array[i];
            String s = str + multiBand.size() + "_" + element.layout;
            int index;
            if ((index = s.indexOf(91)) > 0) {
                s = s.substring(0, index);
            }
            int index2;
            if ((index2 = s.indexOf(40)) > 0) {
                s = s.substring(0, index2);
            }
            if (s.endsWith("H")) {
                s = s.substring(0, s.length() - 1);
            }
            BandStructure.Band obj;
            switch (element.kind) {
                case 1: {
                    obj = this.newElemBand(element, s, multiBand);
                    break;
                }
                case 2: {
                    if (!element.flagTest((byte)2)) {
                        obj = multiBand.newIntBand(s, BandStructure.BCI5);
                        break;
                    }
                    obj = multiBand.newIntBand(s, BandStructure.BRANCH5);
                    break;
                }
                case 3: {
                    obj = multiBand.newIntBand(s, BandStructure.BRANCH5);
                    break;
                }
                case 4: {
                    assert !element.flagTest((byte)1);
                    obj = this.newElemBand(element, s, multiBand);
                    break;
                }
                case 5: {
                    assert !element.flagTest((byte)1);
                    obj = this.newElemBand(element, s, multiBand);
                    this.makeNewAttributeBands(str, element.body, multiBand);
                    break;
                }
                case 7: {
                    obj = this.newElemBand(element, s, multiBand);
                    this.makeNewAttributeBands(str, element.body, multiBand);
                    break;
                }
                case 8: {
                    if (!element.flagTest((byte)8)) {
                        this.makeNewAttributeBands(str, element.body, multiBand);
                        continue;
                    }
                    continue;
                }
                case 6: {
                    obj = multiBand.newCPRefBand(s, BandStructure.UNSIGNED5, element.refKind, element.flagTest((byte)4));
                    break;
                }
                case 9: {
                    continue;
                }
                case 10: {
                    this.makeNewAttributeBands(str, element.body, multiBand);
                    continue;
                }
                default: {
                    assert false;
                    continue;
                }
            }
            if (this.verbose > 1) {
                Utils.log.fine("New attribute band " + obj);
            }
        }
    }
    
    private BandStructure.Band newElemBand(Attribute.Layout.Element element, String s, BandStructure.MultiBand multiBand) {
        if (element.flagTest((byte)1)) {
            return multiBand.newIntBand(s, BandStructure.SIGNED5);
        }
        if (element.len == 1) {
            return multiBand.newIntBand(s, BandStructure.BYTE1);
        }
        return multiBand.newIntBand(s, BandStructure.UNSIGNED5);
    }
    
    protected int setAttributeLayoutIndex(Attribute.Layout obj, int size) {
        int ctype = obj.ctype;
        assert -1 <= size && size < this.attrIndexLimit[ctype];
        List list = (List)this.attrDefs.get(ctype);
        if (size == -1) {
            size = list.size();
            list.add(obj);
            if (this.verbose > 0) {
                Utils.log.info("Adding new attribute at " + obj + ": " + size);
            }
            this.attrIndexTable.put(obj, Integer.valueOf(size));
            return size;
        }
        else {
            if (testBit(this.attrDefSeen[ctype], 1L << size)) {
                throw new RuntimeException("Multiple explicit definition at " + size + ": " + obj);
            }
            long[] attrDefSeen = this.attrDefSeen;
            int n = ctype;
            attrDefSeen[n] |= 1L << size;
            assert 0 <= size && size < this.attrIndexLimit[ctype];
            if (this.verbose > ((this.attrClassFileVersionMask == 0) ? 2 : 0)) {
                Utils.log.fine("Fixing new attribute at " + size + ": " + obj + ((list.get(size) == null) ? "" : ("; replacing " + list.get(size))));
            }
            long[] attrFlagMask = this.attrFlagMask;
            int n2 = ctype;
            attrFlagMask[n2] |= 1L << size;
            this.attrIndexTable.put(list.get(size), null);
            list.set(size, obj);
            this.attrIndexTable.put(obj, Integer.valueOf(size));
            return size;
        }
    }
    
    static int shortCodeHeader(Code code) {
        int max_stack = code.max_stack;
        int max_locals = code.max_locals;
        int length = code.handler_class.length;
        if (length >= BandStructure.shortCodeLimits.length) {
            return 0;
        }
        int argumentSize = code.getMethod().getArgumentSize();
        assert max_locals >= argumentSize;
        if (max_locals < argumentSize) {
            return 0;
        }
        int n = max_locals - argumentSize;
        int n2 = BandStructure.shortCodeLimits[length][0];
        int n3 = BandStructure.shortCodeLimits[length][1];
        if (max_stack >= n2 || n >= n3) {
            return 0;
        }
        int n4 = shortCodeHeader_h_base(length) + (max_stack + n2 * n);
        if (n4 > 255) {
            return 0;
        }
        assert shortCodeHeader_max_stack(n4) == max_stack;
        assert shortCodeHeader_max_na_locals(n4) == n;
        assert shortCodeHeader_handler_count(n4) == length;
        return n4;
    }
    
    static int shortCodeHeader_handler_count(int i) {
        assert i > 0 && i <= 255;
        int n;
        for (n = 0; i >= shortCodeHeader_h_base(n + 1); ++n) {}
        return n;
    }
    
    static int shortCodeHeader_max_stack(int n) {
        int shortCodeHeader_handler_count = shortCodeHeader_handler_count(n);
        return (n - shortCodeHeader_h_base(shortCodeHeader_handler_count)) % BandStructure.shortCodeLimits[shortCodeHeader_handler_count][0];
    }
    
    static int shortCodeHeader_max_na_locals(int n) {
        int shortCodeHeader_handler_count = shortCodeHeader_handler_count(n);
        return (n - shortCodeHeader_h_base(shortCodeHeader_handler_count)) / BandStructure.shortCodeLimits[shortCodeHeader_handler_count][0];
    }
    
    private static int shortCodeHeader_h_base(int n) {
        assert n <= BandStructure.shortCodeLimits.length;
        int n2 = 1;
        for (int i = 0; i < n; ++i) {
            n2 += BandStructure.shortCodeLimits[i][0] * BandStructure.shortCodeLimits[i][1];
        }
        return n2;
    }
    
    protected void putLabel(BandStructure.IntBand intBand, Code code, int n, int n2) {
        intBand.putInt(code.encodeBCI(n2) - code.encodeBCI(n));
    }
    
    protected int getLabel(BandStructure.IntBand intBand, Code code, int n) {
        return code.decodeBCI(intBand.getInt() + code.encodeBCI(n));
    }
    
    protected BandStructure.CPRefBand getCPRefOpBand(int n) {
        switch (Instruction.getCPRefOpTag(n)) {
            case 7: {
                return this.bc_classref;
            }
            case 9: {
                return this.bc_fieldref;
            }
            case 10: {
                return this.bc_methodref;
            }
            case 11: {
                return this.bc_imethodref;
            }
            case 18: {
                return this.bc_indyref;
            }
            case 51: {
                switch (n) {
                    case 234:
                    case 237: {
                        return this.bc_intref;
                    }
                    case 235:
                    case 238: {
                        return this.bc_floatref;
                    }
                    case 20: {
                        return this.bc_longref;
                    }
                    case 239: {
                        return this.bc_doubleref;
                    }
                    case 18:
                    case 19: {
                        return this.bc_stringref;
                    }
                    case 233:
                    case 236: {
                        return this.bc_classref;
                    }
                    case 240:
                    case 241: {
                        return this.bc_loadablevalueref;
                    }
                    default: {
                        break;
                    }
                }
                break;
            }
        }
        assert false;
        return null;
    }
    
    protected BandStructure.CPRefBand selfOpRefBand(int n) {
        assert !(!Instruction.isSelfLinkerOp(n));
        int n2 = n - 202;
        int n3 = n2 >= 14;
        if (!(n3 == 0)) {
            n2 -= 14;
        }
        if (n2 >= 7) {
            n2 -= 7;
        }
        int fieldOp = Instruction.isFieldOp(178 + n2);
        if (n3 == 0) {
            return fieldOp ? this.bc_thisfield : this.bc_thismethod;
        }
        return fieldOp ? this.bc_superfield : this.bc_supermethod;
    }
    
    static OutputStream getDumpStream(BandStructure.Band band, String s) throws IOException {
        return getDumpStream(band.name, band.seqForDebug, s, band);
    }
    
    static OutputStream getDumpStream(ConstantPool.Index index, String s) throws IOException {
        if (index.size() == 0) {
            return new ByteArrayOutputStream();
        }
        return getDumpStream(index.debugName, (int)ConstantPool.TAG_ORDER[index.cpMap[0].tag], s, index);
    }
    
    static OutputStream getDumpStream(String s, int n, String str, Object obj) throws IOException {
        if (BandStructure.dumpDir == null) {
            (BandStructure.dumpDir = File.createTempFile("BD_", "", new File("."))).delete();
            if (BandStructure.dumpDir.mkdir()) {
                Utils.log.info("Dumping bands to " + BandStructure.dumpDir);
            }
        }
        s = s.replace('(', ' ').replace(')', ' ');
        s = s.replace('/', ' ');
        s = s.replace('*', ' ');
        s = s.trim().replace(' ', '_');
        s = (10000 + n + "_" + s).substring(1);
        File file = new File(BandStructure.dumpDir, s + str);
        Utils.log.info("Dumping " + obj + " to " + file);
        return new BufferedOutputStream(new FileOutputStream(file));
    }
    
    static boolean assertCanChangeLength(BandStructure.Band band) {
        switch (band.phase) {
            case 1:
            case 4: {
                return true;
            }
            default: {
                return false;
            }
        }
    }
    
    static boolean assertPhase(BandStructure.Band obj, int i) {
        if (obj.phase() != i) {
            Utils.log.warning("phase expected " + i + " was " + obj.phase() + " in " + obj);
            return false;
        }
        return true;
    }
    
    static int verbose() {
        return Utils.currentPropMap().getInteger("com.sun.java.util.jar.pack.verbose");
    }
    
    static boolean assertPhaseChangeOK(BandStructure.Band band, int n, int i) {
        switch (n * 10 + i) {
            case 1: {
                assert !band.isReader();
                assert band.capacity() >= 0;
                assert band.length() == 0;
                return true;
            }
            case 13:
            case 33: {
                assert band.length() == 0;
                return true;
            }
            case 15:
            case 35: {
                return true;
            }
            case 58: {
                return true;
            }
            case 2: {
                assert !(!band.isReader());
                assert band.capacity() < 0;
                return true;
            }
            case 24: {
                assert Math.max(0, band.capacity()) >= band.valuesExpected();
                assert band.length() <= 0;
                return true;
            }
            case 46: {
                assert band.valuesRemainingForDebug() == band.length();
                return true;
            }
            case 68: {
                assert !(!assertDoneDisbursing(band));
                return true;
            }
            default: {
                if (n == i) {
                    Utils.log.warning("Already in phase " + n);
                }
                else {
                    Utils.log.warning("Unexpected phase " + n + " -> " + i);
                }
                return false;
            }
        }
    }
    
    private static boolean assertDoneDisbursing(BandStructure.Band band) {
        if (band.phase != 6) {
            Utils.log.warning("assertDoneDisbursing: still in phase " + band.phase + ": " + band);
            if (verbose() <= 1) {
                return false;
            }
        }
        int valuesRemainingForDebug = band.valuesRemainingForDebug();
        if (valuesRemainingForDebug > 0) {
            Utils.log.warning("assertDoneDisbursing: " + valuesRemainingForDebug + " values left in " + band);
            if (verbose() <= 1) {
                return false;
            }
        }
        if ((band instanceof BandStructure.MultiBand)) {
            BandStructure.MultiBand multiBand = (BandStructure.MultiBand)band;
            for (int i = 0; i < multiBand.bandCount; ++i) {
                BandStructure.Band obj = multiBand.bands[i];
                if (obj.phase != 8) {
                    Utils.log.warning("assertDoneDisbursing: sub-band still in phase " + obj.phase + ": " + obj);
                    if (verbose() <= 1) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    
    private static void printCDecl(BandStructure.Band band) {
        if ((band instanceof BandStructure.MultiBand)) {
            BandStructure.MultiBand multiBand = (BandStructure.MultiBand)band;
            for (int i = 0; i < multiBand.bandCount; ++i) {
                printCDecl(multiBand.bands[i]);
            }
            return;
        }
        String string = "NULL";
        if ((band instanceof BandStructure.CPRefBand)) {
            ConstantPool.Index index = ((BandStructure.CPRefBand)band).index;
            if (index != null) {
                string = "INDEX(" + index.debugName + ")";
            }
        }
        Coding[] a = { BandStructure.BYTE1, BandStructure.CHAR3, BandStructure.BCI5, BandStructure.BRANCH5, BandStructure.UNSIGNED5, BandStructure.UDELTA5, BandStructure.SIGNED5, BandStructure.DELTA5, BandStructure.MDELTA5 };
        String[] array = { "BYTE1", "CHAR3", "BCI5", "BRANCH5", "UNSIGNED5", "UDELTA5", "SIGNED5", "DELTA5", "MDELTA5" };
        Coding regularCoding = band.regularCoding;
        int index2 = Arrays.asList(a).indexOf(regularCoding);
        String string2;
        if (index2 >= 0) {
            string2 = array[index2];
        }
        else {
            string2 = "CODING" + regularCoding.keyString();
        }
        System.out.println("  BAND_INIT(\"" + band.name() + "\", " + string2 + ", " + string + "),");
        return;
    }
    
    boolean notePrevForAssert(BandStructure.Band band, BandStructure.Band band2) {
        if (this.prevForAssertMap == null) {
            this.prevForAssertMap = new HashMap();
        }
        this.prevForAssertMap.put(band, band2);
        return true;
    }
    
    private boolean assertReadyToReadFrom(BandStructure.Band obj, InputStream inputStream) throws IOException {
        BandStructure.Band obj2 = (BandStructure.Band)this.prevForAssertMap.get(obj);
        if (obj2 != null && phaseCmp(obj2.phase(), 6) < 0) {
            Utils.log.warning("Previous band not done reading.");
            Utils.log.info("    Previous band: " + obj2);
            Utils.log.info("        Next band: " + obj);
            assert this.verbose > 0;
        }
        String access$500 = obj.name;
        if (this.optDebugBands && !access$500.startsWith("(")) {
            assert BandStructure.bandSequenceList != null;
            String str = (String)BandStructure.bandSequenceList.removeFirst();
            if (!str.equals(access$500)) {
                Utils.log.warning("Expected " + access$500 + " but read: " + str);
                return false;
            }
            Utils.log.info("Read band in sequence: " + access$500);
        }
        return true;
    }
    
    private boolean assertValidCPRefs(BandStructure.CPRefBand obj) {
        if (obj.index == null) {
            return true;
        }
        int n = obj.index.size() + 1;
        int i = 0;
        while (i < obj.length()) {
            int valueAtForDebug = obj.valueAtForDebug(i);
            if (valueAtForDebug < 0 || valueAtForDebug >= n) {
                Utils.log.warning("CP ref out of range [" + i + "] = " + valueAtForDebug + " in " + obj);
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private boolean assertReadyToWriteTo(BandStructure.Band obj, OutputStream outputStream) throws IOException {
        BandStructure.Band obj2 = (BandStructure.Band)this.prevForAssertMap.get(obj);
        if (obj2 != null && phaseCmp(obj2.phase(), 8) < 0) {
            Utils.log.warning("Previous band not done writing.");
            Utils.log.info("    Previous band: " + obj2);
            Utils.log.info("        Next band: " + obj);
            assert this.verbose > 0;
        }
        String access$500 = obj.name;
        if (this.optDebugBands && !access$500.startsWith("(")) {
            if (BandStructure.bandSequenceList == null) {
                BandStructure.bandSequenceList = new LinkedList();
            }
            BandStructure.bandSequenceList.add(access$500);
        }
        return true;
    }
    
    protected static boolean testBit(int n, int n2) {
        return (n & n2) != 0x0;
    }
    
    protected static int setBit(int n, int n2, boolean b) {
        return b ? (n | n2) : (n & ~n2);
    }
    
    protected static boolean testBit(long n, long n2) {
        return (n & n2) != 0x0L;
    }
    
    protected static long setBit(long n, long n2, boolean b) {
        return b ? (n | n2) : (n & ~n2);
    }
    
    static void printArrayTo(PrintStream printStream, int[] array, int n, int n2) {
        for (int n3 = n2 - n, i = 0; i < n3; ++i) {
            if (i % 10 == 0) {
                printStream.println();
            }
            else {
                printStream.print(" ");
            }
            printStream.print(array[n + i]);
        }
        printStream.println();
    }
    
    static void printArrayTo(PrintStream printStream, ConstantPool.Entry[] array, int n, int n2) {
        printArrayTo(printStream, array, n, n2, false);
    }
    
    static void printArrayTo(PrintStream printStream, ConstantPool.Entry[] array, int n, int n2, boolean b) {
        StringBuffer x = new StringBuffer();
        for (int n3 = n2 - n, i = 0; i < n3; ++i) {
            ConstantPool.Entry entry = array[n + i];
            printStream.print(n + i);
            printStream.print("=");
            if (b) {
                printStream.print((int)entry.tag);
                printStream.print(":");
            }
            String stringValue = entry.stringValue();
            x.setLength(0);
            for (int j = 0; j < stringValue.length(); ++j) {
                int char1 = stringValue.charAt(j);
                if (char1 >= ' ' && char1 <= '~' && char1 != '\\') {
                    x.append((char)char1);
                }
                else if (char1 == '\\') {
                    x.append("\\\\");
                }
                else if (char1 == '\n') {
                    x.append("\\n");
                }
                else if (char1 == '\t') {
                    x.append("\\t");
                }
                else if (char1 == '\r') {
                    x.append("\\r");
                }
                else {
                    String string = "000" + Integer.toHexString(char1);
                    x.append("\\u").append(string.substring(string.length() - 4));
                }
            }
            printStream.println(x);
        }
    }
    
    protected static Object[] realloc(Object[] array, int n) {
        Object[] array2 = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), n);
        System.arraycopy(array, 0, array2, 0, Math.min(array.length, n));
        return array2;
    }
    
    protected static Object[] realloc(Object[] array) {
        return realloc(array, Math.max(10, array.length * 2));
    }
    
    protected static int[] realloc(int[] array, int b) {
        if (b == 0) {
            return Constants.noInts;
        }
        if (array == null) {
            return new int[b];
        }
        int[] array2 = new int[b];
        System.arraycopy(array, 0, array2, 0, Math.min(array.length, b));
        return array2;
    }
    
    protected static int[] realloc(int[] array) {
        return realloc(array, Math.max(10, array.length * 2));
    }
    
    protected static byte[] realloc(byte[] array, int b) {
        if (b == 0) {
            return Constants.noBytes;
        }
        if (array == null) {
            return new byte[b];
        }
        byte[] array2 = new byte[b];
        System.arraycopy(array, 0, array2, 0, Math.min(array.length, b));
        return array2;
    }
    
    protected static byte[] realloc(byte[] array) {
        return realloc(array, Math.max(10, array.length * 2));
    }
    
    static boolean access$100(BandStructure bandStructure, BandStructure.Band band, OutputStream outputStream) throws IOException {
        return bandStructure.assertReadyToWriteTo(band, outputStream);
    }
    
    static boolean access$200(BandStructure bandStructure, BandStructure.Band band, InputStream inputStream) throws IOException {
        return bandStructure.assertReadyToReadFrom(band, inputStream);
    }
    
    static boolean access$400(BandStructure bandStructure, BandStructure.CPRefBand cpRefBand) {
        return bandStructure.assertValidCPRefs(cpRefBand);
    }
    
    static {
        BYTE1 = Coding.of(1, 256);
        CHAR3 = Coding.of(3, 128);
        BCI5 = Coding.of(5, 4);
        BRANCH5 = Coding.of(5, 4, 2);
        UNSIGNED5 = Coding.of(5, 64);
        UDELTA5 = BandStructure.UNSIGNED5.getDeltaCoding();
        SIGNED5 = Coding.of(5, 64, 1);
        DELTA5 = BandStructure.SIGNED5.getDeltaCoding();
        MDELTA5 = Coding.of(5, 64, 2).getDeltaCoding();
        basicCodings = new Coding[] { null, Coding.of(1, 256, 0), Coding.of(1, 256, 1), Coding.of(1, 256, 0).getDeltaCoding(), Coding.of(1, 256, 1).getDeltaCoding(), Coding.of(2, 256, 0), Coding.of(2, 256, 1), Coding.of(2, 256, 0).getDeltaCoding(), Coding.of(2, 256, 1).getDeltaCoding(), Coding.of(3, 256, 0), Coding.of(3, 256, 1), Coding.of(3, 256, 0).getDeltaCoding(), Coding.of(3, 256, 1).getDeltaCoding(), Coding.of(4, 256, 0), Coding.of(4, 256, 1), Coding.of(4, 256, 0).getDeltaCoding(), Coding.of(4, 256, 1).getDeltaCoding(), Coding.of(5, 4, 0), Coding.of(5, 4, 1), Coding.of(5, 4, 2), Coding.of(5, 16, 0), Coding.of(5, 16, 1), Coding.of(5, 16, 2), Coding.of(5, 32, 0), Coding.of(5, 32, 1), Coding.of(5, 32, 2), Coding.of(5, 64, 0), Coding.of(5, 64, 1), Coding.of(5, 64, 2), Coding.of(5, 128, 0), Coding.of(5, 128, 1), Coding.of(5, 128, 2), Coding.of(5, 4, 0).getDeltaCoding(), Coding.of(5, 4, 1).getDeltaCoding(), Coding.of(5, 4, 2).getDeltaCoding(), Coding.of(5, 16, 0).getDeltaCoding(), Coding.of(5, 16, 1).getDeltaCoding(), Coding.of(5, 16, 2).getDeltaCoding(), Coding.of(5, 32, 0).getDeltaCoding(), Coding.of(5, 32, 1).getDeltaCoding(), Coding.of(5, 32, 2).getDeltaCoding(), Coding.of(5, 64, 0).getDeltaCoding(), Coding.of(5, 64, 1).getDeltaCoding(), Coding.of(5, 64, 2).getDeltaCoding(), Coding.of(5, 128, 0).getDeltaCoding(), Coding.of(5, 128, 1).getDeltaCoding(), Coding.of(5, 128, 2).getDeltaCoding(), Coding.of(2, 192, 0), Coding.of(2, 224, 0), Coding.of(2, 240, 0), Coding.of(2, 248, 0), Coding.of(2, 252, 0), Coding.of(2, 8, 0).getDeltaCoding(), Coding.of(2, 8, 1).getDeltaCoding(), Coding.of(2, 16, 0).getDeltaCoding(), Coding.of(2, 16, 1).getDeltaCoding(), Coding.of(2, 32, 0).getDeltaCoding(), Coding.of(2, 32, 1).getDeltaCoding(), Coding.of(2, 64, 0).getDeltaCoding(), Coding.of(2, 64, 1).getDeltaCoding(), Coding.of(2, 128, 0).getDeltaCoding(), Coding.of(2, 128, 1).getDeltaCoding(), Coding.of(2, 192, 0).getDeltaCoding(), Coding.of(2, 192, 1).getDeltaCoding(), Coding.of(2, 224, 0).getDeltaCoding(), Coding.of(2, 224, 1).getDeltaCoding(), Coding.of(2, 240, 0).getDeltaCoding(), Coding.of(2, 240, 1).getDeltaCoding(), Coding.of(2, 248, 0).getDeltaCoding(), Coding.of(2, 248, 1).getDeltaCoding(), Coding.of(3, 192, 0), Coding.of(3, 224, 0), Coding.of(3, 240, 0), Coding.of(3, 248, 0), Coding.of(3, 252, 0), Coding.of(3, 8, 0).getDeltaCoding(), Coding.of(3, 8, 1).getDeltaCoding(), Coding.of(3, 16, 0).getDeltaCoding(), Coding.of(3, 16, 1).getDeltaCoding(), Coding.of(3, 32, 0).getDeltaCoding(), Coding.of(3, 32, 1).getDeltaCoding(), Coding.of(3, 64, 0).getDeltaCoding(), Coding.of(3, 64, 1).getDeltaCoding(), Coding.of(3, 128, 0).getDeltaCoding(), Coding.of(3, 128, 1).getDeltaCoding(), Coding.of(3, 192, 0).getDeltaCoding(), Coding.of(3, 192, 1).getDeltaCoding(), Coding.of(3, 224, 0).getDeltaCoding(), Coding.of(3, 224, 1).getDeltaCoding(), Coding.of(3, 240, 0).getDeltaCoding(), Coding.of(3, 240, 1).getDeltaCoding(), Coding.of(3, 248, 0).getDeltaCoding(), Coding.of(3, 248, 1).getDeltaCoding(), Coding.of(4, 192, 0), Coding.of(4, 224, 0), Coding.of(4, 240, 0), Coding.of(4, 248, 0), Coding.of(4, 252, 0), Coding.of(4, 8, 0).getDeltaCoding(), Coding.of(4, 8, 1).getDeltaCoding(), Coding.of(4, 16, 0).getDeltaCoding(), Coding.of(4, 16, 1).getDeltaCoding(), Coding.of(4, 32, 0).getDeltaCoding(), Coding.of(4, 32, 1).getDeltaCoding(), Coding.of(4, 64, 0).getDeltaCoding(), Coding.of(4, 64, 1).getDeltaCoding(), Coding.of(4, 128, 0).getDeltaCoding(), Coding.of(4, 128, 1).getDeltaCoding(), Coding.of(4, 192, 0).getDeltaCoding(), Coding.of(4, 192, 1).getDeltaCoding(), Coding.of(4, 224, 0).getDeltaCoding(), Coding.of(4, 224, 1).getDeltaCoding(), Coding.of(4, 240, 0).getDeltaCoding(), Coding.of(4, 240, 1).getDeltaCoding(), Coding.of(4, 248, 0).getDeltaCoding(), Coding.of(4, 248, 1).getDeltaCoding(), null };
        assert BandStructure.basicCodings[0] == null;
        assert BandStructure.basicCodings[1] != null;
        assert BandStructure.basicCodings[115] != null;
        HashMap<Coding, Integer> basicCodingIndexes2 = new HashMap();
        for (int i = 0; i < BandStructure.basicCodings.length; ++i) {
            Coding coding = BandStructure.basicCodings[i];
            if (coding != null) {
                assert i >= 1;
                assert i <= 115;
                basicCodingIndexes2.put(coding, Integer.valueOf(i));
            }
        }
        basicCodingIndexes = basicCodingIndexes2;
        defaultMetaCoding = new byte[] { 0 };
        noMetaCoding = new byte[0];
        boolean n = false;
        assert !((n = true) == 0);
        if (n != 0) {
            for (int j = 0; j < BandStructure.basicCodings.length; ++j) {
                Coding coding2 = BandStructure.basicCodings[j];
                if (coding2 != null && coding2.B() != 1 && coding2.L() != 0) {
                    for (int k = 0; k <= 255; ++k) {
                        encodeEscapeValue(k, coding2);
                    }
                }
            }
        }
        shortCodeLimits = new int[][] { { 12, 12 }, { 8, 8 }, { 7, 7 } };
        BandStructure.dumpDir = null;
        BandStructure.bandSequenceList = null;
        return;
    }
    
    abstract class Band
    {
        private int phase;
        private final String name;
        private int valuesExpected;
        protected long outputSize;
        public final Coding regularCoding;
        public final int seqForDebug;
        public int elementCountForDebug;
        protected int lengthForDebug;
        
        protected Band(String s, Coding regularCoding) {
            this.phase = 0;
            this.outputSize = -1L;
            this.lengthForDebug = -1;
            this.name = s;
            this.regularCoding = regularCoding;
            this.seqForDebug = ++BandStructure.nextSeqForDebug;
            if (BandStructure.this.verbose > 2) {
                Utils.log.fine("Band " + this.seqForDebug + " is " + s);
            }
        }
        
        public BandStructure.Band init() {
            if (BandStructure.this.isReader) {
                this.readyToExpect();
                return this;
            }
            this.readyToCollect();
            return this;
        }
        
        boolean isReader() {
            return BandStructure.this.isReader;
        }
        
        int phase() {
            return this.phase;
        }
        
        String name() {
            return this.name;
        }
        
        public abstract int capacity();
        
        protected abstract void setCapacity(int p0);
        
        public abstract int length();
        
        protected abstract int valuesRemainingForDebug();
        
        public final int valuesExpected() {
            return this.valuesExpected;
        }
        
        public final void writeTo(OutputStream outputStream) throws IOException {
            assert !(!BandStructure.access$100(BandStructure.this, this, outputStream));
            this.setPhase(5);
            this.writeDataTo(outputStream);
            this.doneWriting();
            return;
        }
        
        abstract void chooseBandCodings() throws IOException;
        
        public final long outputSize() {
            if (this.outputSize >= 0L) {
                long outputSize = this.outputSize;
                assert outputSize == this.computeOutputSize();
                return outputSize;
            }
            else {
                return this.computeOutputSize();
            }
        }
        
        protected abstract long computeOutputSize();
        
        protected abstract void writeDataTo(OutputStream p0) throws IOException;
        
        void expectLength(int valuesExpected) {
            assert !(!BandStructure.assertPhase(this, 2));
            assert this.valuesExpected == 0;
            assert valuesExpected >= 0;
            this.valuesExpected = valuesExpected;
            return;
        }
        
        void expectMoreLength(int n) {
            assert !(!BandStructure.assertPhase(this, 2));
            this.valuesExpected += n;
            return;
        }
        
        private void readyToCollect() {
            this.setCapacity(1);
            this.setPhase(1);
        }
        
        protected void doneWriting() {
            assert !(!BandStructure.assertPhase(this, 5));
            this.setPhase(8);
            return;
        }
        
        private void readyToExpect() {
            this.setPhase(2);
        }
        
        public final void readFrom(InputStream inputStream) throws IOException {
            assert !(!BandStructure.access$200(BandStructure.this, this, inputStream));
            this.setCapacity(this.valuesExpected());
            this.setPhase(4);
            this.readDataFrom(inputStream);
            this.readyToDisburse();
            return;
        }
        
        protected abstract void readDataFrom(InputStream p0) throws IOException;
        
        protected void readyToDisburse() {
            if (BandStructure.this.verbose > 1) {
                Utils.log.fine("readyToDisburse " + this);
            }
            this.setPhase(6);
        }
        
        public void doneDisbursing() {
            assert !(!BandStructure.assertPhase(this, 6));
            this.setPhase(8);
            return;
        }
        
        public final void doneWithUnusedBand() {
            if (BandStructure.this.isReader) {
                assert !(!BandStructure.assertPhase(this, 2));
                assert this.valuesExpected() == 0;
                this.setPhase(4);
                this.setPhase(6);
                this.setPhase(8);
            }
            else {
                this.setPhase(3);
            }
        }
        
        protected void setPhase(int phase) {
            assert !(!BandStructure.assertPhaseChangeOK(this, this.phase, phase));
            this.phase = phase;
            return;
        }
        
        @Override
        public String toString() {
            int i = (this.lengthForDebug != -1) ? this.lengthForDebug : this.length();
            String s = this.name;
            if (i != 0) {
                s = s + "[" + i + "]";
            }
            if (this.elementCountForDebug != 0) {
                s = s + "(" + this.elementCountForDebug + ")";
                return s;
            }
            return s;
        }
    }
    
    class ValueBand extends BandStructure.Band
    {
        private int[] values;
        private int length;
        private int valuesDisbursed;
        private CodingMethod bandCoding;
        private byte[] metaCoding;
        static final /* synthetic */ boolean $assertionsDisabled;
        
        protected ValueBand(String s, Coding coding) {
            super(s, coding);
        }
        
        @Override
        public int capacity() {
            return (this.values == null) ? -1 : this.values.length;
        }
        
        @Override
        protected void setCapacity(int n) {
            assert this.length <= n;
            if (n == -1) {
                this.values = null;
                return;
            }
            this.values = BandStructure.realloc(this.values, n);
            return;
        }
        
        @Override
        public int length() {
            return this.length;
        }
        
        @Override
        protected int valuesRemainingForDebug() {
            return this.length - this.valuesDisbursed;
        }
        
        protected int valueAtForDebug(int n) {
            return this.values[n];
        }
        
        void patchValue(int n, int n2) {
            assert this == BandStructure.this.archive_header_S;
            assert n == 1;
            assert n < this.length;
            this.values[n] = n2;
            this.outputSize = -1L;
            return;
        }
        
        protected void initializeValues(int[] values) {
            assert !(!BandStructure.assertCanChangeLength(this));
            assert this.length == 0;
            this.values = values;
            this.length = values.length;
            return;
        }
        
        protected void addValue(int n) {
            assert !(!BandStructure.assertCanChangeLength(this));
            if (this.length == this.values.length) {
                this.setCapacity((this.length < 1000) ? (this.length * 10) : (this.length * 2));
            }
            this.values[this.length++] = n;
            return;
        }
        
        private boolean canVaryCoding() {
            return BandStructure.this.optVaryCodings && this.length != 0 && this != BandStructure.this.archive_header_0 && this != BandStructure.this.archive_header_S && this != BandStructure.this.archive_header_1 && (this.regularCoding.min() <= -256 || this.regularCoding.max() >= 256);
        }
        
        private boolean shouldVaryCoding() {
            assert !(!this.canVaryCoding());
            return BandStructure.this.effort >= 9 || this.length >= 100;
        }
        
        protected void chooseBandCodings() throws IOException {
            int canVaryCoding = this.canVaryCoding();
            if (canVaryCoding == 0 || !this.shouldVaryCoding()) {
                if (this.regularCoding.canRepresent(this.values, 0, this.length)) {
                    this.bandCoding = this.regularCoding;
                }
                else {
                    assert !(canVaryCoding == 0);
                    if (BandStructure.this.verbose > 1) {
                        Utils.log.fine("regular coding fails in band " + this.name());
                    }
                    this.bandCoding = BandStructure.UNSIGNED5;
                }
                this.outputSize = -1L;
            }
            else {
                int[] array = { 0, 0 };
                this.bandCoding = BandStructure.this.chooseCoding(this.values, 0, this.length, this.regularCoding, this.name(), array);
                this.outputSize = (long)array[0];
                if (this.outputSize == 0L) {
                    this.outputSize = -1L;
                }
            }
            if (this.bandCoding != this.regularCoding) {
                this.metaCoding = this.bandCoding.getMetaCoding(this.regularCoding);
                if (BandStructure.this.verbose > 1) {
                    Utils.log.fine("alternate coding " + this + " " + this.bandCoding);
                }
            }
            else if (canVaryCoding && BandStructure.decodeEscapeValue(this.values[0], this.regularCoding) >= 0) {
                this.metaCoding = BandStructure.defaultMetaCoding;
            }
            else {
                this.metaCoding = BandStructure.noMetaCoding;
            }
            if (this.metaCoding.length > 0 && (BandStructure.this.verbose > 2 || (BandStructure.this.verbose > 1 && this.metaCoding.length > 1))) {
                StringBuffer obj = new StringBuffer();
                for (int i = 0; i < this.metaCoding.length; ++i) {
                    if (i == 1) {
                        obj.append(" /");
                    }
                    obj.append(" ").append(this.metaCoding[i] & 0xFF);
                }
                Utils.log.fine("   meta-coding " + obj);
            }
            assert this.outputSize == (long)((Coding)this.bandCoding).getLength(this.values, 0, this.length) : this.bandCoding + " : " + this.outputSize + " != " + ((Coding)this.bandCoding).getLength(this.values, 0, this.length) + " ?= " + BandStructure.this.getCodingChooser().computeByteSize(this.bandCoding, this.values, 0, this.length);
            if (this.metaCoding.length > 0) {
                if (this.outputSize >= 0L) {
                    this.outputSize += (long)this.computeEscapeSize();
                }
                for (int j = 1; j < this.metaCoding.length; ++j) {
                    BandStructure.this.band_headers.putByte(this.metaCoding[j] & 0xFF);
                }
            }
            return;
        }
        
        @Override
        protected long computeOutputSize() {
            this.outputSize = (long)BandStructure.this.getCodingChooser().computeByteSize(this.bandCoding, this.values, 0, this.length);
            assert this.outputSize < 2147483647L;
            return this.outputSize += (long)this.computeEscapeSize();
        }
        
        protected int computeEscapeSize() {
            if (this.metaCoding.length == 0) {
                return 0;
            }
            return this.regularCoding.setD(0).getLength(BandStructure.encodeEscapeValue(this.metaCoding[0] & 0xFF, this.regularCoding));
        }
        
        @Override
        protected void writeDataTo(OutputStream outputStream) throws IOException {
            if (this.length == 0) {
                return;
            }
            long count = 0L;
            if (outputStream == BandStructure.this.outputCounter) {
                count = BandStructure.this.outputCounter.getCount();
            }
            if (this.metaCoding.length > 0) {
                this.regularCoding.setD(0).writeTo(outputStream, BandStructure.encodeEscapeValue(this.metaCoding[0] & 0xFF, this.regularCoding));
            }
            this.bandCoding.writeArrayTo(outputStream, this.values, 0, this.length);
            if (outputStream == BandStructure.this.outputCounter && !BandStructure.ValueBand.$assertionsDisabled && this.outputSize != BandStructure.this.outputCounter.getCount() - count) {
                throw new AssertionError(this.outputSize + " != " + BandStructure.this.outputCounter.getCount() + "-" + count);
            }
            if (BandStructure.this.optDumpBands) {
                this.dumpBand();
            }
            return;
        }
        
        @Override
        protected void readDataFrom(InputStream inputStream) throws IOException {
            this.length = this.valuesExpected();
            if (this.length == 0) {
                return;
            }
            if (BandStructure.this.verbose > 1) {
                Utils.log.fine("Reading band " + this);
            }
            if (!this.canVaryCoding()) {
                this.bandCoding = this.regularCoding;
                this.metaCoding = BandStructure.noMetaCoding;
            }
            else {
                assert !(!inputStream.markSupported());
                inputStream.mark(5);
                int from = this.regularCoding.setD(0).readFrom(inputStream);
                int decodeEscapeValue = BandStructure.decodeEscapeValue(from, this.regularCoding);
                if (decodeEscapeValue < 0) {
                    inputStream.reset();
                    this.bandCoding = this.regularCoding;
                    this.metaCoding = BandStructure.noMetaCoding;
                }
                else if (decodeEscapeValue == 0) {
                    this.bandCoding = this.regularCoding;
                    this.metaCoding = BandStructure.defaultMetaCoding;
                }
                else {
                    if (BandStructure.this.verbose > 2) {
                        Utils.log.fine("found X=" + from + " => XB=" + decodeEscapeValue);
                    }
                    this.bandCoding = BandStructure.this.getBandHeader(decodeEscapeValue, this.regularCoding);
                    int bandHeaderBytePos0 = BandStructure.this.bandHeaderBytePos0;
                    this.metaCoding = new byte[BandStructure.this.bandHeaderBytePos - bandHeaderBytePos0];
                    System.arraycopy(BandStructure.this.bandHeaderBytes, bandHeaderBytePos0, this.metaCoding, 0, this.metaCoding.length);
                }
            }
            if (this.bandCoding != this.regularCoding && BandStructure.this.verbose > 1) {
                Utils.log.fine(this.name() + ": irregular coding " + this.bandCoding);
            }
            this.bandCoding.readArrayFrom(inputStream, this.values, 0, this.length);
            if (BandStructure.this.optDumpBands) {
                this.dumpBand();
            }
            return;
        }
        
        @Override
        public void doneDisbursing() {
            super.doneDisbursing();
            this.values = null;
        }
        
        private void dumpBand() throws IOException {
            assert !(!BandStructure.this.optDumpBands);
            try (final PrintStream printStream = new PrintStream(BandStructure.getDumpStream(this, ".txt"))) {
                printStream.print("# length=" + this.length + " size=" + this.outputSize() + ((this.bandCoding == this.regularCoding) ? "" : " irregular") + " coding=" + this.bandCoding);
                if (this.metaCoding != BandStructure.noMetaCoding) {
                    StringBuffer obj = new StringBuffer();
                    for (int i = 0; i < this.metaCoding.length; ++i) {
                        if (i == 1) {
                            obj.append(" /");
                        }
                        obj.append(" ").append(this.metaCoding[i] & 0xFF);
                    }
                    printStream.print(" //header: " + obj);
                }
                BandStructure.printArrayTo(printStream, this.values, 0, this.length);
            }
            try (final OutputStream dumpStream = BandStructure.getDumpStream(this, ".bnd")) {
                this.bandCoding.writeArrayTo(dumpStream, this.values, 0, this.length);
            }
            return;
        }
        
        protected int getValue() {
            assert this.phase() == 6;
            if (BandStructure.this.optDebugBands && this.length == 0 && this.valuesDisbursed == this.length) {
                return 0;
            }
            assert this.valuesDisbursed <= this.length;
            return this.values[this.valuesDisbursed++];
        }
        
        public void resetForSecondPass() {
            assert this.phase() == 6;
            assert this.valuesDisbursed == this.length();
            this.valuesDisbursed = 0;
            return;
        }
    }
    
    class ByteBand extends BandStructure.Band
    {
        private ByteArrayOutputStream bytes;
        private ByteArrayOutputStream bytesForDump;
        private InputStream in;
        
        public ByteBand(String s) {
            super(s, BandStructure.BYTE1);
        }
        
        @Override
        public int capacity() {
            return (this.bytes == null) ? -1 : Integer.MAX_VALUE;
        }
        
        @Override
        protected void setCapacity(int size) {
            assert this.bytes == null;
            this.bytes = new ByteArrayOutputStream(size);
            return;
        }
        
        public void destroy() {
            this.lengthForDebug = this.length();
            this.bytes = null;
        }
        
        @Override
        public int length() {
            return (this.bytes == null) ? -1 : this.bytes.size();
        }
        
        public void reset() {
            this.bytes.reset();
        }
        
        @Override
        protected int valuesRemainingForDebug() {
            return (this.bytes == null) ? -1 : ((ByteArrayInputStream)this.in).available();
        }
        
        protected void chooseBandCodings() throws IOException {
            assert BandStructure.decodeEscapeValue(this.regularCoding.min(), this.regularCoding) < 0;
            assert BandStructure.decodeEscapeValue(this.regularCoding.max(), this.regularCoding) < 0;
            return;
        }
        
        @Override
        protected long computeOutputSize() {
            return (long)this.bytes.size();
        }
        
        public void writeDataTo(OutputStream out) throws IOException {
            if (this.length() == 0) {
                return;
            }
            this.bytes.writeTo(out);
            if (BandStructure.this.optDumpBands) {
                this.dumpBand();
            }
            this.destroy();
            return;
        }
        
        private void dumpBand() throws IOException {
            assert !(!BandStructure.this.optDumpBands);
            try (final OutputStream dumpStream = BandStructure.getDumpStream(this, ".bnd")) {
                if (this.bytesForDump != null) {
                    this.bytesForDump.writeTo(dumpStream);
                }
                else {
                    this.bytes.writeTo(dumpStream);
                }
            }
            return;
        }
        
        public void readDataFrom(InputStream inputStream) throws IOException {
            int i = this.valuesExpected();
            if (i == 0) {
                return;
            }
            if (BandStructure.this.verbose > 1) {
                this.lengthForDebug = i;
                Utils.log.fine("Reading band " + this);
                this.lengthForDebug = -1;
            }
            byte[] array = new byte[Math.min(i, 16384)];
            while (i > 0) {
                int read = inputStream.read(array, 0, Math.min(i, array.length));
                if (read < 0) {
                    throw new EOFException();
                }
                this.bytes.write(array, 0, read);
                i -= read;
            }
            if (BandStructure.this.optDumpBands) {
                this.dumpBand();
            }
            return;
        }
        
        public void readyToDisburse() {
            this.in = new ByteArrayInputStream(this.bytes.toByteArray());
            super.readyToDisburse();
        }
        
        @Override
        public void doneDisbursing() {
            super.doneDisbursing();
            if (BandStructure.this.optDumpBands && this.bytesForDump != null && this.bytesForDump.size() > 0) {
                try {
                    this.dumpBand();
                }
                catch (IOException cause) {
                    throw new RuntimeException(cause);
                }
            }
            this.in = null;
            this.bytes = null;
            this.bytesForDump = null;
        }
        
        public void setInputStreamFrom(InputStream in) throws IOException {
            assert this.bytes == null;
            assert !(!BandStructure.access$200(BandStructure.this, this, in));
            this.setPhase(4);
            this.in = in;
            if (BandStructure.this.optDumpBands) {
                this.bytesForDump = new ByteArrayOutputStream();
                this.in = new FilterInputStream(in) {
                    @Override
                    public int read() throws IOException {
                        int read = this.in.read();
                        if (read >= 0) {
                            BandStructure.ByteBand.this.bytesForDump.write(read);
                        }
                        return read;
                    }
                    
                    @Override
                    public int read(byte[] array, int n, int len) throws IOException {
                        int read = this.in.read(array, n, len);
                        if (read >= 0) {
                            BandStructure.ByteBand.this.bytesForDump.write(array, n, read);
                        }
                        return read;
                    }
                };
            }
            super.readyToDisburse();
            return;
        }
        
        public OutputStream collectorStream() {
            assert this.phase() == 1;
            assert this.bytes != null;
            return this.bytes;
        }
        
        public InputStream getInputStream() {
            assert this.phase() == 6;
            assert this.in != null;
            return this.in;
        }
        
        public int getByte() throws IOException {
            int read = this.getInputStream().read();
            if (read < 0) {
                throw new EOFException();
            }
            return read;
        }
        
        public void putByte(int n) throws IOException {
            assert n == (n & 0xFF);
            this.collectorStream().write(n);
            return;
        }
        
        @Override
        public String toString() {
            return "byte " + super.toString();
        }
    }
    
    class IntBand extends BandStructure.ValueBand
    {
        public IntBand(String s, Coding coding) {
            super(s, coding);
        }
        
        public void putInt(int n) {
            assert this.phase() == 1;
            this.addValue(n);
            return;
        }
        
        public int getInt() {
            return this.getValue();
        }
        
        public int getIntTotal() {
            assert this.phase() == 6;
            assert this.valuesRemainingForDebug() == this.length();
            int n = 0;
            for (int i = this.length(); i > 0; --i) {
                n += this.getInt();
            }
            this.resetForSecondPass();
            return n;
        }
        
        public int getIntCount(int n) {
            assert this.phase() == 6;
            assert this.valuesRemainingForDebug() == this.length();
            int n2 = 0;
            for (int i = this.length(); i > 0; --i) {
                if (this.getInt() == n) {
                    ++n2;
                }
            }
            this.resetForSecondPass();
            return n2;
        }
    }
    
    class CPRefBand extends BandStructure.ValueBand
    {
        ConstantPool.Index index;
        boolean nullOK;
        
        public CPRefBand(String s, Coding coding, byte b, boolean nullOK) {
            super(s, coding);
            this.nullOK = nullOK;
            if (b != 0) {
                BandStructure.this.setBandIndex(this, b);
            }
        }
        
        public CPRefBand(BandStructure bandStructure, String s, Coding coding, byte b) {
            this(bandStructure, s, coding, b, false);
        }
        
        public CPRefBand(BandStructure bandStructure, String s, Coding coding, Object o) {
            this(bandStructure, s, coding, (byte)0, false);
        }
        
        public void setIndex(ConstantPool.Index index) {
            this.index = index;
        }
        
        @Override
        protected void readDataFrom(InputStream inputStream) throws IOException {
            super.readDataFrom(inputStream);
            assert !(!BandStructure.access$400(BandStructure.this, this));
            return;
        }
        
        public void putRef(ConstantPool.Entry entry) {
            this.addValue(this.encodeRefOrNull(entry, this.index));
        }
        
        public void putRef(ConstantPool.Entry entry, ConstantPool.Index index) {
            assert this.index == null;
            this.addValue(this.encodeRefOrNull(entry, index));
            return;
        }
        
        public void putRef(ConstantPool.Entry entry, byte b) {
            this.putRef(entry, BandStructure.this.getCPIndex(b));
        }
        
        public ConstantPool.Entry getRef() {
            if (this.index == null) {
                Utils.log.warning("No index for " + this);
            }
            assert this.index != null;
            return this.decodeRefOrNull(this.getValue(), this.index);
        }
        
        public ConstantPool.Entry getRef(ConstantPool.Index index) {
            assert this.index == null;
            return this.decodeRefOrNull(this.getValue(), index);
        }
        
        public ConstantPool.Entry getRef(byte b) {
            return this.getRef(BandStructure.this.getCPIndex(b));
        }
        
        private int encodeRefOrNull(ConstantPool.Entry entry, ConstantPool.Index index) {
            int encodeRef;
            if (entry == null) {
                encodeRef = -1;
            }
            else {
                encodeRef = BandStructure.this.encodeRef(entry, index);
            }
            return (this.nullOK ? 1 : 0) + encodeRef;
        }
        
        private ConstantPool.Entry decodeRefOrNull(int n, ConstantPool.Index index) {
            int n2 = n - (this.nullOK ? 1 : 0);
            if (n2 == -1) {
                return null;
            }
            return BandStructure.this.decodeRef(n2, index);
        }
    }
    
    class MultiBand extends BandStructure.Band
    {
        BandStructure.Band[] bands;
        int bandCount;
        private int cap;
        
        MultiBand(String s, Coding coding) {
            super(s, coding);
            this.bands = new BandStructure.Band[10];
            this.bandCount = 0;
            this.cap = -1;
        }
        
        @Override
        public BandStructure.Band init() {
            super.init();
            this.setCapacity(0);
            if (this.phase() == 2) {
                this.setPhase(4);
                this.setPhase(6);
                return this;
            }
            return this;
        }
        
        int size() {
            return this.bandCount;
        }
        
        BandStructure.Band get(int n) {
            assert n < this.bandCount;
            return this.bands[n];
        }
        
        BandStructure.Band[] toArray() {
            return (BandStructure.Band[])(BandStructure.Band[])BandStructure.realloc(this.bands, this.bandCount);
        }
        
        void add(BandStructure.Band band) {
            assert !(!BandStructure.this.notePrevForAssert(band, this.bands[this.bandCount - 1]));
            if (this.bandCount == this.bands.length) {
                this.bands = (BandStructure.Band[])(BandStructure.Band[])BandStructure.realloc(this.bands);
            }
            this.bands[this.bandCount++] = band;
            return;
        }
        
        BandStructure.ByteBand newByteBand(String s) {
            BandStructure.ByteBand byteBand = new BandStructure.ByteBand(s);
            byteBand.init();
            this.add(byteBand);
            return byteBand;
        }
        
        BandStructure.IntBand newIntBand(String s) {
            BandStructure.IntBand intBand = new BandStructure.IntBand(s, this.regularCoding);
            intBand.init();
            this.add(intBand);
            return intBand;
        }
        
        BandStructure.IntBand newIntBand(String s, Coding coding) {
            BandStructure.IntBand intBand = new BandStructure.IntBand(s, coding);
            intBand.init();
            this.add(intBand);
            return intBand;
        }
        
        BandStructure.MultiBand newMultiBand(String s, Coding coding) {
            BandStructure.MultiBand multiBand = new BandStructure.MultiBand(s, coding);
            multiBand.init();
            this.add(multiBand);
            return multiBand;
        }
        
        BandStructure.CPRefBand newCPRefBand(String s, byte b) {
            BandStructure.CPRefBand cpRefBand = new BandStructure.CPRefBand(s, this.regularCoding, b);
            cpRefBand.init();
            this.add(cpRefBand);
            return cpRefBand;
        }
        
        BandStructure.CPRefBand newCPRefBand(String s, Coding coding, byte b) {
            BandStructure.CPRefBand cpRefBand = new BandStructure.CPRefBand(s, coding, b);
            cpRefBand.init();
            this.add(cpRefBand);
            return cpRefBand;
        }
        
        BandStructure.CPRefBand newCPRefBand(String s, Coding coding, byte b, boolean b2) {
            BandStructure.CPRefBand cpRefBand = new BandStructure.CPRefBand(s, coding, b, b2);
            cpRefBand.init();
            this.add(cpRefBand);
            return cpRefBand;
        }
        
        int bandCount() {
            return this.bandCount;
        }
        
        @Override
        public int capacity() {
            return this.cap;
        }
        
        public void setCapacity(int cap) {
            this.cap = cap;
        }
        
        @Override
        public int length() {
            return 0;
        }
        
        public int valuesRemainingForDebug() {
            return 0;
        }
        
        protected void chooseBandCodings() throws IOException {
            for (int i = 0; i < this.bandCount; ++i) {
                this.bands[i].chooseBandCodings();
            }
        }
        
        @Override
        protected long computeOutputSize() {
            long n = 0L;
            int i = 0;
            while (i < this.bandCount) {
                BandStructure.Band detailMessage = this.bands[i];
                long outputSize = detailMessage.outputSize();
                assert outputSize >= 0L : detailMessage;
                n += outputSize;
                ++i;
            }
            return n;
        }
        
        @Override
        protected void writeDataTo(OutputStream outputStream) throws IOException {
            long count = 0L;
            if (BandStructure.this.outputCounter != null) {
                count = BandStructure.this.outputCounter.getCount();
            }
            for (int i = 0; i < this.bandCount; ++i) {
                BandStructure.Band obj = this.bands[i];
                obj.writeTo(outputStream);
                if (BandStructure.this.outputCounter != null) {
                    long count2 = BandStructure.this.outputCounter.getCount();
                    long lng = count2 - count;
                    count = count2;
                    if ((BandStructure.this.verbose > 0 && lng > 0L) || BandStructure.this.verbose > 1) {
                        Utils.log.info("  ...wrote " + lng + " bytes from " + obj);
                    }
                }
            }
        }
        
        @Override
        protected void readDataFrom(InputStream inputStream) throws IOException {
            assert false;
            for (int i = 0; i < this.bandCount; ++i) {
                BandStructure.Band obj = this.bands[i];
                obj.readFrom(inputStream);
                if ((BandStructure.this.verbose > 0 && obj.length() > 0) || BandStructure.this.verbose > 1) {
                    Utils.log.info("  ...read " + obj);
                }
            }
            return;
        }
        
        @Override
        public String toString() {
            return "{" + this.bandCount() + " bands: " + super.toString() + "}";
        }
    }
    
    private static class ByteCounter extends FilterOutputStream
    {
        private long count;
        
        public ByteCounter(OutputStream out) {
            super(out);
        }
        
        public long getCount() {
            return this.count;
        }
        
        public void setCount(long count) {
            this.count = count;
        }
        
        @Override
        public void write(int n) throws IOException {
            ++this.count;
            if (this.out != null) {
                this.out.write(n);
            }
        }
        
        @Override
        public void write(byte[] b, int off, int len) throws IOException {
            this.count += (long)len;
            if (this.out != null) {
                this.out.write(b, off, len);
            }
        }
        
        @Override
        public String toString() {
            return String.valueOf(this.getCount());
        }
    }
}
